import pytest

import stripe


class TestAPIResource(object):
    class MyResource(stripe.APIResource):
        OBJECT_NAME = "myresource"

    class MyDeletableResource(stripe.DeletableAPIResource):
        OBJECT_NAME = "myresource"

        @classmethod
        def my_method(cls, **params):
            return cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            )

        @classmethod
        async def my_method_async(cls, **params):
            return cls.construct_from(
                await cls._static_request_async(
                    "post",
                    cls.class_url(),
                    params=params,
                ),
                stripe.api_key,
            )

    def test_retrieve_and_refresh(self, http_client_mock):
        path = "/v1/myresources/foo%2A"
        query_string = "myparam=5"
        key = "sk_test_123"
        stripe_version = "2018-02-28"
        stripe_account = "acct_foo"
        http_client_mock.stub_request(
            "get",
            path,
            query_string,
            '{"id": "foo2", "bobble": "scrobble"}',
            rheaders={"request-id": "req_id"},
        )

        res = self.MyResource.retrieve(
            "foo*",
            myparam=5,
            stripe_version="2018-02-28",
            stripe_account="acct_foo",
        )

        http_client_mock.assert_requested(
            "get",
            path=path,
            query_string=query_string,
            api_key=key,
            stripe_version=stripe_version,
            stripe_account=stripe_account,
        )
        assert res.bobble == "scrobble"
        assert res.id == "foo2"
        assert res.api_key == key
        assert res.stripe_version == stripe_version
        assert res.stripe_account == stripe_account

        assert res.last_response is not None
        assert res.last_response.request_id == "req_id"

        path = "/v1/myresources/foo2"
        query_string = "myparam=5"
        http_client_mock.stub_request(
            "get", path, query_string, '{"frobble": 5}'
        )

        res = res.refresh()

        http_client_mock.assert_requested(
            "get",
            path=path,
            query_string=query_string,
            api_key=key,
            stripe_version=stripe_version,
            stripe_account=stripe_account,
        )
        assert res.frobble == 5
        with pytest.raises(KeyError):
            res["bobble"]

    def test_convert_to_stripe_object(self):
        sample = {
            "foo": "bar",
            "adict": {"object": "charge", "id": 42, "amount": 7},
            "alist": [{"object": "customer", "name": "chilango"}],
        }

        converted = stripe.util.convert_to_stripe_object(
            sample, "akey", None, None
        )

        # Types
        assert isinstance(converted, stripe.stripe_object.StripeObject)
        assert isinstance(converted.adict, stripe.Charge)
        assert len(converted.alist) == 1
        assert isinstance(converted.alist[0], stripe.Customer)

        # Values
        assert converted.foo == "bar"
        assert converted.adict.id == 42
        assert converted.alist[0].name == "chilango"

        # Stripping
        # TODO: We should probably be stripping out this property
        # self.assertRaises(AttributeError, getattr, converted.adict, 'object')

    def test_raise_on_incorrect_id_type(self):
        for obj in [None, 1, 3.14, dict(), list(), set(), tuple(), object()]:
            with pytest.raises(stripe.error.InvalidRequestError):
                self.MyResource.retrieve(obj)

    def test_class_methods_use_global_options(self, http_client_mock):
        key = "newkey"
        stripe_version = "2023-01-01"
        stripe_account = "acct_foo"

        resource = self.MyDeletableResource.construct_from(
            {"id": "foo"},
            key=key,
            stripe_version=stripe_version,
            stripe_account=stripe_account,
        )

        http_client_mock.stub_request(
            "get",
            "/v1/myresources/foo",
            rbody='{"id": "foo"}',
        )

        resource.retrieve("foo")

        http_client_mock.assert_requested(
            "get",
            path="/v1/myresources/foo",
            api_key=stripe.api_key,
            stripe_version=stripe.api_version,
            extra_headers={"Stripe-Account": None},
        )

        http_client_mock.stub_request(
            "post",
            "/v1/myresources",
            rbody='{"id": "foo", "object": "myresource"}',
        )

        self.MyDeletableResource.my_method()

        http_client_mock.assert_requested(
            "post",
            path="/v1/myresources",
            api_key=stripe.api_key,
            stripe_version=stripe.api_version,
            extra_headers={"Stripe-Account": None},
        )

    def test_class_method_prefers_method_arguments(self, http_client_mock):
        http_client_mock.stub_request(
            "get",
            "/v1/myresources/foo",
            rbody='{"id": "foo"}',
        )

        resource = self.MyResource.construct_from(
            {"id": "foo"},
            key="oldkey",
            stripe_version="2000-01-01",
            stripe_account="acct_foo",
        )

        resource.retrieve(
            "foo",
            api_key="newkey",
            stripe_version="2023-01-01",
            stripe_account="acct_bar",
        )

        http_client_mock.assert_requested(
            "get",
            path="/v1/myresources/foo",
            api_key="newkey",
            stripe_version="2023-01-01",
            stripe_account="acct_bar",
        )

    def test_retrieve_forwards_options(self, http_client_mock):
        http_client_mock.stub_request(
            "get",
            "/v1/myresources/foo",
            rbody='{"id": "foo"}',
        )

        res = self.MyDeletableResource.retrieve(
            "foo",
            api_key="newkey",
            stripe_version="2023-01-01",
            stripe_account="foo",
        )

        http_client_mock.assert_requested(
            "get",
            path="/v1/myresources/foo",
            api_key="newkey",
            stripe_version="2023-01-01",
            stripe_account="foo",
        )

        http_client_mock.stub_request(
            "delete",
            "/v1/myresources/foo",
        )

        res.delete()

        http_client_mock.assert_requested(
            "delete",
            path="/v1/myresources/foo",
            api_key="newkey",
            stripe_version="2023-01-01",
            stripe_account="foo",
        )

    def test_class_method_forwards_options(self, http_client_mock):
        from stripe._object_classes import OBJECT_CLASSES

        OBJECT_CLASSES["myresource"] = self.MyDeletableResource

        http_client_mock.stub_request(
            "post",
            "/v1/myresources",
            rbody='{"id": "foo", "object": "myresource"}',
        )

        res = self.MyDeletableResource.my_method(
            api_key="newkey", stripe_version="2023-01-01", stripe_account="foo"
        )

        http_client_mock.assert_requested(
            "post",
            path="/v1/myresources",
            api_key="newkey",
            stripe_version="2023-01-01",
            stripe_account="foo",
        )

        http_client_mock.stub_request(
            "delete",
            "/v1/myresources/foo",
        )

        res.delete()

        http_client_mock.assert_requested(
            "delete",
            path="/v1/myresources/foo",
            api_key="newkey",
            stripe_version="2023-01-01",
            stripe_account="foo",
        )

        del OBJECT_CLASSES["myresource"]

    def test_instance_method_forwards_options(self, http_client_mock):
        http_client_mock.stub_request(
            "get",
            "/v1/myresources/foo",
            rbody='{"id": "foo"}',
        )

        key = "newkey"
        stripe_version = "2023-01-01"
        stripe_account = "acct_foo"

        resource = self.MyResource.construct_from(
            {"id": "foo"},
            key=key,
            stripe_version=stripe_version,
            stripe_account=stripe_account,
        )

        resource.refresh()

        http_client_mock.assert_requested(
            "get",
            path="/v1/myresources/foo",
            api_key=key,
            stripe_version=stripe_version,
            stripe_account=stripe_account,
        )

    def test_instance_method_prefers_method_arguments(self, http_client_mock):
        class MyDeletableResource(stripe.DeletableAPIResource):
            OBJECT_NAME = "mydeletableresource"

        http_client_mock.stub_request(
            "delete",
            "/v1/mydeletableresources/foo",
            rbody='{"id": "foo"}',
        )

        resource = MyDeletableResource.construct_from(
            {"id": "foo"},
            key="oldkey",
            stripe_version="2000-01-01",
            stripe_account="acct_foo",
        )

        resource.delete(
            api_key="newkey",
            stripe_version="2023-01-01",
            stripe_account="acct_bar",
        )

        http_client_mock.assert_requested(
            "delete",
            path="/v1/mydeletableresources/foo",
            api_key="newkey",
            stripe_version="2023-01-01",
            stripe_account="acct_bar",
        )

    @pytest.mark.anyio
    async def test_async_methods_succeed(self, http_client_mock):
        http_client_mock.stub_request(
            "post",
            "/v1/myresources",
            rbody='{"id": "foo", "object": "myresource"}',
        )
        resource = await self.MyDeletableResource.my_method_async()
        http_client_mock.assert_requested(
            "post",
            path="/v1/myresources",
        )

        http_client_mock.stub_request(
            "get",
            "/v1/myresources/foo",
            rbody='{"id": "foo", "object": "myresource"}',
        )
        await resource.refresh_async()
        http_client_mock.assert_requested(
            "get",
            path="/v1/myresources/foo",
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._request_options import RequestOptions
from stripe._updateable_api_resource import UpdateableAPIResource
from stripe._util import sanitize_id
from typing import ClassVar, Dict, List, cast
from typing_extensions import Literal, NotRequired, Unpack


class Feature(
    CreateableAPIResource["Feature"],
    ListableAPIResource["Feature"],
    UpdateableAPIResource["Feature"],
):
    """
    A feature represents a monetizable ability or functionality in your system.
    Features can be assigned to products, and when those products are purchased, Stripe will create an entitlement to the feature for the purchasing customer.
    """

    OBJECT_NAME: ClassVar[Literal["entitlements.feature"]] = (
        "entitlements.feature"
    )

    class CreateParams(RequestOptions):
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        lookup_key: str
        """
        A unique key you provide as your own system identifier. This may be up to 80 characters.
        """
        metadata: NotRequired[Dict[str, str]]
        """
        Set of key-value pairs that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
        """
        name: str
        """
        The feature's name, for your own purpose, not meant to be displayable to the customer.
        """

    class ListParams(RequestOptions):
        archived: NotRequired[bool]
        """
        If set, filter results to only include features with the given archive status.
        """
        ending_before: NotRequired[str]
        """
        A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
        """
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        limit: NotRequired[int]
        """
        A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
        """
        lookup_key: NotRequired[str]
        """
        If set, filter results to only include features with the given lookup_key.
        """
        starting_after: NotRequired[str]
        """
        A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
        """

    class ModifyParams(RequestOptions):
        active: NotRequired[bool]
        """
        Inactive features cannot be attached to new products and will not be returned from the features list endpoint.
        """
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        metadata: NotRequired["Literal['']|Dict[str, str]"]
        """
        Set of key-value pairs that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
        """
        name: NotRequired[str]
        """
        The feature's name, for your own purpose, not meant to be displayable to the customer.
        """

    class RetrieveParams(RequestOptions):
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """

    active: bool
    """
    Inactive features cannot be attached to new products and will not be returned from the features list endpoint.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    lookup_key: str
    """
    A unique key you provide as your own system identifier. This may be up to 80 characters.
    """
    metadata: Dict[str, str]
    """
    Set of key-value pairs that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    name: str
    """
    The feature's name, for your own purpose, not meant to be displayable to the customer.
    """
    object: Literal["entitlements.feature"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """

    @classmethod
    def create(cls, **params: Unpack["Feature.CreateParams"]) -> "Feature":
        """
        Creates a feature
        """
        return cast(
            "Feature",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["Feature.CreateParams"]
    ) -> "Feature":
        """
        Creates a feature
        """
        return cast(
            "Feature",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def list(
        cls, **params: Unpack["Feature.ListParams"]
    ) -> ListObject["Feature"]:
        """
        Retrieve a list of features
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["Feature.ListParams"]
    ) -> ListObject["Feature"]:
        """
        Retrieve a list of features
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def modify(
        cls, id: str, **params: Unpack["Feature.ModifyParams"]
    ) -> "Feature":
        """
        Update a feature's metadata or permanently deactivate it.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "Feature",
            cls._static_request(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    async def modify_async(
        cls, id: str, **params: Unpack["Feature.ModifyParams"]
    ) -> "Feature":
        """
        Update a feature's metadata or permanently deactivate it.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "Feature",
            await cls._static_request_async(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["Feature.RetrieveParams"]
    ) -> "Feature":
        """
        Retrieves a feature
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["Feature.RetrieveParams"]
    ) -> "Feature":
        """
        Retrieves a feature
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance



# using global variables
import stripe  # noqa: IMP101
from stripe._base_address import BaseAddresses

from typing import Optional


class RequestorOptions(object):
    api_key: Optional[str]
    stripe_account: Optional[str]
    stripe_version: Optional[str]
    base_addresses: BaseAddresses
    max_network_retries: Optional[int]

    def __init__(
        self,
        api_key: Optional[str] = None,
        stripe_account: Optional[str] = None,
        stripe_version: Optional[str] = None,
        base_addresses: BaseAddresses = {},
        max_network_retries: Optional[int] = None,
    ):
        self.api_key = api_key
        self.stripe_account = stripe_account
        self.stripe_version = stripe_version
        self.base_addresses = {}

        # Base addresses can be unset (for correct merging).
        # If they are not set, then we will use default API bases defined on stripe.
        if base_addresses.get("api"):
            self.base_addresses["api"] = base_addresses.get("api")
        if base_addresses.get("connect") is not None:
            self.base_addresses["connect"] = base_addresses.get("connect")
        if base_addresses.get("files") is not None:
            self.base_addresses["files"] = base_addresses.get("files")

        self.max_network_retries = max_network_retries

    def to_dict(self):
        """
        Returns a dict representation of the object.
        """
        return {
            "api_key": self.api_key,
            "stripe_account": self.stripe_account,
            "stripe_version": self.stripe_version,
            "base_addresses": self.base_addresses,
            "max_network_retries": self.max_network_retries,
        }


class _GlobalRequestorOptions(RequestorOptions):
    def __init__(self):
        pass

    @property
    def base_addresses(self):
        return {
            "api": stripe.api_base,
            "connect": stripe.connect_api_base,
            "files": stripe.upload_api_base,
        }

    @property
    def api_key(self):
        return stripe.api_key

    @property
    def stripe_version(self):
        return stripe.api_version

    @property
    def stripe_account(self):
        return None

    @property
    def max_network_retries(self):
        return stripe.max_network_retries



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._api_resource import APIResource
from stripe._request_options import RequestOptions
from stripe._stripe_object import StripeObject
from typing import ClassVar, List, Optional, cast
from typing_extensions import Literal, NotRequired, Unpack


class Association(APIResource["Association"]):
    """
    A Tax Association exposes the Tax Transactions that Stripe attempted to create on your behalf based on the PaymentIntent input
    """

    OBJECT_NAME: ClassVar[Literal["tax.association"]] = "tax.association"

    class StatusDetails(StripeObject):
        class Committed(StripeObject):
            class Reversal(StripeObject):
                class StatusDetails(StripeObject):
                    class Committed(StripeObject):
                        transaction: str
                        """
                        The [Tax Transaction](https://stripe.com/docs/api/tax/transaction/object)
                        """

                    class Errored(StripeObject):
                        reason: Literal[
                            "original_transaction_voided",
                            "unique_reference_violation",
                        ]
                        """
                        Details on why we could not commit the reversal Tax Transaction
                        """
                        refund_id: str
                        """
                        The [Refund](https://stripe.com/docs/api/refunds/object) ID that should have created a tax reversal.
                        """

                    committed: Optional[Committed]
                    errored: Optional[Errored]
                    _inner_class_types = {
                        "committed": Committed,
                        "errored": Errored,
                    }

                status: Literal["committed", "errored"]
                """
                Status of the attempted Tax Transaction reversal.
                """
                status_details: StatusDetails
                _inner_class_types = {"status_details": StatusDetails}

            reversals: List[Reversal]
            """
            Attempts to create Tax Transaction reversals
            """
            transaction: str
            """
            The [Tax Transaction](https://stripe.com/docs/api/tax/transaction/object)
            """
            _inner_class_types = {"reversals": Reversal}

        class Errored(StripeObject):
            reason: Literal[
                "another_payment_associated_with_calculation",
                "calculation_expired",
                "currency_mismatch",
                "unique_reference_violation",
            ]
            """
            Details on why we could not commit the Tax Transaction
            """

        committed: Optional[Committed]
        errored: Optional[Errored]
        _inner_class_types = {"committed": Committed, "errored": Errored}

    class FindParams(RequestOptions):
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        payment_intent: str
        """
        Valid [PaymentIntent](https://stripe.com/docs/api/payment_intents/object) id
        """

    calculation: str
    """
    The [Tax Calculation](https://stripe.com/docs/api/tax/calculations/object) that was included in PaymentIntent.
    """
    id: str
    """
    Unique identifier for the object.
    """
    object: Literal["tax.association"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    payment_intent: str
    """
    The [PaymentIntent](https://stripe.com/docs/api/payment_intents/object) that this Tax Association is tracking.
    """
    status: Literal["committed", "errored"]
    """
    Status of the Tax Association.
    """
    status_details: StatusDetails

    @classmethod
    def find(cls, **params: Unpack["Association.FindParams"]) -> "Association":
        """
        Finds a tax association object by PaymentIntent id.
        """
        return cast(
            "Association",
            cls._static_request(
                "get",
                "/v1/tax/associations/find",
                params=params,
            ),
        )

    @classmethod
    async def find_async(
        cls, **params: Unpack["Association.FindParams"]
    ) -> "Association":
        """
        Finds a tax association object by PaymentIntent id.
        """
        return cast(
            "Association",
            await cls._static_request_async(
                "get",
                "/v1/tax/associations/find",
                params=params,
            ),
        )

    _inner_class_types = {"status_details": StatusDetails}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._request_options import RequestOptions
from stripe._stripe_object import StripeObject
from typing import ClassVar, List, Optional
from typing_extensions import Literal, NotRequired, Unpack


class FinancingTransaction(ListableAPIResource["FinancingTransaction"]):
    """
    This is an object representing the details of a transaction on a Capital financing object.
    """

    OBJECT_NAME: ClassVar[Literal["capital.financing_transaction"]] = (
        "capital.financing_transaction"
    )

    class Details(StripeObject):
        class Transaction(StripeObject):
            charge: Optional[str]
            """
            The linked payment ID.
            """
            treasury_transaction: Optional[str]
            """
            The linked Treasury Financing Transaction ID.
            """

        advance_amount: int
        """
        The advance amount being repaid, paid out, or reversed in minor units.
        """
        currency: str
        """
        The currency of the financing transaction.
        """
        fee_amount: int
        """
        The fee amount being repaid, paid out, or reversed in minor units.
        """
        linked_payment: Optional[str]
        """
        The linked payment for the transaction. This field only applies to financing transactions of type `paydown` and reason `automatic_withholding`.
        """
        reason: Optional[
            Literal[
                "automatic_withholding",
                "automatic_withholding_refund",
                "collection",
                "collection_failure",
                "financing_cancellation",
                "refill",
                "requested_by_user",
                "user_initiated",
            ]
        ]
        """
        The reason for the financing transaction (if applicable).
        """
        reversed_transaction: Optional[str]
        """
        The reversed transaction. This field only applies to financing
        transactions of type `reversal`.
        """
        total_amount: int
        """
        The advance and fee amount being repaid, paid out, or reversed in minor units.
        """
        transaction: Optional[Transaction]
        """
        This is an object representing a linked transaction on a Capital Financing Transaction.
        """
        _inner_class_types = {"transaction": Transaction}

    class ListParams(RequestOptions):
        charge: NotRequired[str]
        """
        For transactions of type `paydown` and reason `automatic_withholding` only, only returns transactions that were created as a result of this charge.
        """
        ending_before: NotRequired[str]
        """
        A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
        """
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        financing_offer: NotRequired[str]
        """
        Returns transactions that were created that apply to this financing offer ID.
        """
        limit: NotRequired[int]
        """
        A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
        """
        reversed_transaction: NotRequired[str]
        """
        Only returns transactions that are responsible for reversing this financing transaction ID.
        """
        starting_after: NotRequired[str]
        """
        A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
        """
        treasury_transaction: NotRequired[str]
        """
        For transactions of type `paydown` and reason `automatic_withholding` only, only returns transactions that were created as a result of this Treasury Transaction.
        """

    class RetrieveParams(RequestOptions):
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """

    account: str
    """
    The ID of the merchant associated with this financing transaction.
    """
    created_at: int
    """
    Time at which the financing transaction was created. Given in seconds since unix epoch.
    """
    details: Details
    """
    This is an object representing a transaction on a Capital financing offer.
    """
    financing_offer: Optional[str]
    """
    The Capital financing offer for this financing transaction.
    """
    id: str
    """
    A unique identifier for the financing transaction object.
    """
    legacy_balance_transaction_source: Optional[str]
    """
    The Capital transaction object that predates the Financing Transactions API and
    corresponds with the balance transaction that was created as a result of this
    financing transaction.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["capital.financing_transaction"]
    """
    The object type: financing_transaction
    """
    type: Literal["payment", "payout", "reversal"]
    """
    The type of the financing transaction.
    """
    user_facing_description: Optional[str]
    """
    A human-friendly description of the financing transaction.
    """

    @classmethod
    def list(
        cls, **params: Unpack["FinancingTransaction.ListParams"]
    ) -> ListObject["FinancingTransaction"]:
        """
        Returns a list of financing transactions. The transactions are returned in sorted order,
        with the most recent transactions appearing first.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["FinancingTransaction.ListParams"]
    ) -> ListObject["FinancingTransaction"]:
        """
        Returns a list of financing transactions. The transactions are returned in sorted order,
        with the most recent transactions appearing first.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["FinancingTransaction.RetrieveParams"]
    ) -> "FinancingTransaction":
        """
        Retrieves a financing transaction for a financing offer.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["FinancingTransaction.RetrieveParams"]
    ) -> "FinancingTransaction":
        """
        Retrieves a financing transaction for a financing offer.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {"details": Details}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._api_resource import APIResource
from stripe._expandable_field import ExpandableField
from stripe._request_options import RequestOptions
from stripe._stripe_object import StripeObject
from typing import ClassVar, List, Optional
from typing_extensions import Literal, NotRequired, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._payment_method import PaymentMethod


class Mandate(APIResource["Mandate"]):
    """
    A Mandate is a record of the permission that your customer gives you to debit their payment method.
    """

    OBJECT_NAME: ClassVar[Literal["mandate"]] = "mandate"

    class CustomerAcceptance(StripeObject):
        class Offline(StripeObject):
            pass

        class Online(StripeObject):
            ip_address: Optional[str]
            """
            The customer accepts the mandate from this IP address.
            """
            user_agent: Optional[str]
            """
            The customer accepts the mandate using the user agent of the browser.
            """

        accepted_at: Optional[int]
        """
        The time that the customer accepts the mandate.
        """
        offline: Optional[Offline]
        online: Optional[Online]
        type: Literal["offline", "online"]
        """
        The mandate includes the type of customer acceptance information, such as: `online` or `offline`.
        """
        _inner_class_types = {"offline": Offline, "online": Online}

    class MultiUse(StripeObject):
        pass

    class PaymentMethodDetails(StripeObject):
        class AcssDebit(StripeObject):
            default_for: Optional[List[Literal["invoice", "subscription"]]]
            """
            List of Stripe products where this mandate can be selected automatically.
            """
            interval_description: Optional[str]
            """
            Description of the interval. Only required if the 'payment_schedule' parameter is 'interval' or 'combined'.
            """
            payment_schedule: Literal["combined", "interval", "sporadic"]
            """
            Payment schedule for the mandate.
            """
            transaction_type: Literal["business", "personal"]
            """
            Transaction type of the mandate.
            """

        class AmazonPay(StripeObject):
            pass

        class AuBecsDebit(StripeObject):
            url: str
            """
            The URL of the mandate. This URL generally contains sensitive information about the customer and should be shared with them exclusively.
            """

        class BacsDebit(StripeObject):
            network_status: Literal[
                "accepted", "pending", "refused", "revoked"
            ]
            """
            The status of the mandate on the Bacs network. Can be one of `pending`, `revoked`, `refused`, or `accepted`.
            """
            reference: str
            """
            The unique reference identifying the mandate on the Bacs network.
            """
            revocation_reason: Optional[
                Literal[
                    "account_closed",
                    "bank_account_restricted",
                    "bank_ownership_changed",
                    "could_not_process",
                    "debit_not_authorized",
                ]
            ]
            """
            When the mandate is revoked on the Bacs network this field displays the reason for the revocation.
            """
            url: str
            """
            The URL that will contain the mandate that the customer has signed.
            """

        class Card(StripeObject):
            pass

        class Cashapp(StripeObject):
            pass

        class Link(StripeObject):
            pass

        class Paypal(StripeObject):
            billing_agreement_id: Optional[str]
            """
            The PayPal Billing Agreement ID (BAID). This is an ID generated by PayPal which represents the mandate between the merchant and the customer.
            """
            fingerprint: Optional[str]
            """
            Uniquely identifies this particular PayPal account. You can use this attribute to check whether two PayPal accounts are the same.
            """
            payer_id: Optional[str]
            """
            PayPal account PayerID. This identifier uniquely identifies the PayPal customer.
            """
            verified_email: Optional[str]
            """
            Owner's verified email. Values are verified or provided by PayPal directly
            (if supported) at the time of authorization or settlement. They cannot be set or mutated.
            """

        class Payto(StripeObject):
            amount: Optional[int]
            """
            Amount that will be collected. It is required when `amount_type` is `fixed`.
            """
            amount_type: Literal["fixed", "maximum"]
            """
            The type of amount that will be collected. The amount charged must be exact or up to the value of `amount` param for `fixed` or `maximum` type respectively.
            """
            end_date: Optional[str]
            """
            Date, in YYYY-MM-DD format, after which payments will not be collected. Defaults to no end date.
            """
            payment_schedule: Literal[
                "adhoc",
                "annual",
                "daily",
                "fortnightly",
                "monthly",
                "quarterly",
                "semi_annual",
                "weekly",
            ]
            """
            The periodicity at which payments will be collected.
            """
            payments_per_period: Optional[int]
            """
            The number of payments that will be made during a payment period. Defaults to 1 except for when `payment_schedule` is `adhoc`. In that case, it defaults to no limit.
            """
            purpose: Optional[
                Literal[
                    "dependant_support",
                    "government",
                    "loan",
                    "mortgage",
                    "other",
                    "pension",
                    "personal",
                    "retail",
                    "salary",
                    "tax",
                    "utility",
                ]
            ]
            """
            The purpose for which payments are made. Defaults to retail.
            """
            start_date: Optional[str]
            """
            Date, in YYYY-MM-DD format, from which payments will be collected. Defaults to confirmation time.
            """

        class RevolutPay(StripeObject):
            pass

        class SepaDebit(StripeObject):
            reference: str
            """
            The unique reference of the mandate.
            """
            url: str
            """
            The URL of the mandate. This URL generally contains sensitive information about the customer and should be shared with them exclusively.
            """

        class UsBankAccount(StripeObject):
            collection_method: Optional[Literal["paper"]]
            """
            Mandate collection method
            """

        acss_debit: Optional[AcssDebit]
        amazon_pay: Optional[AmazonPay]
        au_becs_debit: Optional[AuBecsDebit]
        bacs_debit: Optional[BacsDebit]
        card: Optional[Card]
        cashapp: Optional[Cashapp]
        link: Optional[Link]
        paypal: Optional[Paypal]
        payto: Optional[Payto]
        revolut_pay: Optional[RevolutPay]
        sepa_debit: Optional[SepaDebit]
        type: str
        """
        This mandate corresponds with a specific payment method type. The `payment_method_details` includes an additional hash with the same name and contains mandate information that's specific to that payment method.
        """
        us_bank_account: Optional[UsBankAccount]
        _inner_class_types = {
            "acss_debit": AcssDebit,
            "amazon_pay": AmazonPay,
            "au_becs_debit": AuBecsDebit,
            "bacs_debit": BacsDebit,
            "card": Card,
            "cashapp": Cashapp,
            "link": Link,
            "paypal": Paypal,
            "payto": Payto,
            "revolut_pay": RevolutPay,
            "sepa_debit": SepaDebit,
            "us_bank_account": UsBankAccount,
        }

    class SingleUse(StripeObject):
        amount: int
        """
        The amount of the payment on a single use mandate.
        """
        currency: str
        """
        The currency of the payment on a single use mandate.
        """

    class RetrieveParams(RequestOptions):
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """

    customer_acceptance: CustomerAcceptance
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    multi_use: Optional[MultiUse]
    object: Literal["mandate"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    on_behalf_of: Optional[str]
    """
    The account (if any) that the mandate is intended for.
    """
    payment_method: ExpandableField["PaymentMethod"]
    """
    ID of the payment method associated with this mandate.
    """
    payment_method_details: PaymentMethodDetails
    single_use: Optional[SingleUse]
    status: Literal["active", "inactive", "pending"]
    """
    The mandate status indicates whether or not you can use it to initiate a payment.
    """
    type: Literal["multi_use", "single_use"]
    """
    The type of the mandate.
    """

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["Mandate.RetrieveParams"]
    ) -> "Mandate":
        """
        Retrieves a Mandate object.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["Mandate.RetrieveParams"]
    ) -> "Mandate":
        """
        Retrieves a Mandate object.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {
        "customer_acceptance": CustomerAcceptance,
        "multi_use": MultiUse,
        "payment_method_details": PaymentMethodDetails,
        "single_use": SingleUse,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from stripe.issuing._authorization import Authorization
from typing import List, cast
from typing_extensions import Literal, NotRequired, TypedDict


class AuthorizationService(StripeService):
    class CaptureParams(TypedDict):
        capture_amount: NotRequired[int]
        """
        The amount to capture from the authorization. If not provided, the full amount of the authorization will be captured. This amount is in the authorization currency and in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
        """
        close_authorization: NotRequired[bool]
        """
        Whether to close the authorization after capture. Defaults to true. Set to false to enable multi-capture flows.
        """
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        purchase_details: NotRequired[
            "AuthorizationService.CaptureParamsPurchaseDetails"
        ]
        """
        Additional purchase information that is optionally provided by the merchant.
        """

    class CaptureParamsPurchaseDetails(TypedDict):
        fleet: NotRequired[
            "AuthorizationService.CaptureParamsPurchaseDetailsFleet"
        ]
        """
        Fleet-specific information for transactions using Fleet cards.
        """
        flight: NotRequired[
            "AuthorizationService.CaptureParamsPurchaseDetailsFlight"
        ]
        """
        Information about the flight that was purchased with this transaction.
        """
        fuel: NotRequired[
            "AuthorizationService.CaptureParamsPurchaseDetailsFuel"
        ]
        """
        Information about fuel that was purchased with this transaction.
        """
        lodging: NotRequired[
            "AuthorizationService.CaptureParamsPurchaseDetailsLodging"
        ]
        """
        Information about lodging that was purchased with this transaction.
        """
        receipt: NotRequired[
            List["AuthorizationService.CaptureParamsPurchaseDetailsReceipt"]
        ]
        """
        The line items in the purchase.
        """
        reference: NotRequired[str]
        """
        A merchant-specific order number.
        """

    class CaptureParamsPurchaseDetailsFleet(TypedDict):
        cardholder_prompt_data: NotRequired[
            "AuthorizationService.CaptureParamsPurchaseDetailsFleetCardholderPromptData"
        ]
        """
        Answers to prompts presented to the cardholder at the point of sale. Prompted fields vary depending on the configuration of your physical fleet cards. Typical points of sale support only numeric entry.
        """
        purchase_type: NotRequired[
            Literal[
                "fuel_and_non_fuel_purchase",
                "fuel_purchase",
                "non_fuel_purchase",
            ]
        ]
        """
        The type of purchase. One of `fuel_purchase`, `non_fuel_purchase`, or `fuel_and_non_fuel_purchase`.
        """
        reported_breakdown: NotRequired[
            "AuthorizationService.CaptureParamsPurchaseDetailsFleetReportedBreakdown"
        ]
        """
        More information about the total amount. This information is not guaranteed to be accurate as some merchants may provide unreliable data.
        """
        service_type: NotRequired[
            Literal["full_service", "non_fuel_transaction", "self_service"]
        ]
        """
        The type of fuel service. One of `non_fuel_transaction`, `full_service`, or `self_service`.
        """

    class CaptureParamsPurchaseDetailsFleetCardholderPromptData(TypedDict):
        driver_id: NotRequired[str]
        """
        Driver ID.
        """
        odometer: NotRequired[int]
        """
        Odometer reading.
        """
        unspecified_id: NotRequired[str]
        """
        An alphanumeric ID. This field is used when a vehicle ID, driver ID, or generic ID is entered by the cardholder, but the merchant or card network did not specify the prompt type.
        """
        user_id: NotRequired[str]
        """
        User ID.
        """
        vehicle_number: NotRequired[str]
        """
        Vehicle number.
        """

    class CaptureParamsPurchaseDetailsFleetReportedBreakdown(TypedDict):
        fuel: NotRequired[
            "AuthorizationService.CaptureParamsPurchaseDetailsFleetReportedBreakdownFuel"
        ]
        """
        Breakdown of fuel portion of the purchase.
        """
        non_fuel: NotRequired[
            "AuthorizationService.CaptureParamsPurchaseDetailsFleetReportedBreakdownNonFuel"
        ]
        """
        Breakdown of non-fuel portion of the purchase.
        """
        tax: NotRequired[
            "AuthorizationService.CaptureParamsPurchaseDetailsFleetReportedBreakdownTax"
        ]
        """
        Information about tax included in this transaction.
        """

    class CaptureParamsPurchaseDetailsFleetReportedBreakdownFuel(TypedDict):
        gross_amount_decimal: NotRequired[str]
        """
        Gross fuel amount that should equal Fuel Volume multipled by Fuel Unit Cost, inclusive of taxes.
        """

    class CaptureParamsPurchaseDetailsFleetReportedBreakdownNonFuel(TypedDict):
        gross_amount_decimal: NotRequired[str]
        """
        Gross non-fuel amount that should equal the sum of the line items, inclusive of taxes.
        """

    class CaptureParamsPurchaseDetailsFleetReportedBreakdownTax(TypedDict):
        local_amount_decimal: NotRequired[str]
        """
        Amount of state or provincial Sales Tax included in the transaction amount. Null if not reported by merchant or not subject to tax.
        """
        national_amount_decimal: NotRequired[str]
        """
        Amount of national Sales Tax or VAT included in the transaction amount. Null if not reported by merchant or not subject to tax.
        """

    class CaptureParamsPurchaseDetailsFlight(TypedDict):
        departure_at: NotRequired[int]
        """
        The time that the flight departed.
        """
        passenger_name: NotRequired[str]
        """
        The name of the passenger.
        """
        refundable: NotRequired[bool]
        """
        Whether the ticket is refundable.
        """
        segments: NotRequired[
            List[
                "AuthorizationService.CaptureParamsPurchaseDetailsFlightSegment"
            ]
        ]
        """
        The legs of the trip.
        """
        travel_agency: NotRequired[str]
        """
        The travel agency that issued the ticket.
        """

    class CaptureParamsPurchaseDetailsFlightSegment(TypedDict):
        arrival_airport_code: NotRequired[str]
        """
        The three-letter IATA airport code of the flight's destination.
        """
        carrier: NotRequired[str]
        """
        The airline carrier code.
        """
        departure_airport_code: NotRequired[str]
        """
        The three-letter IATA airport code that the flight departed from.
        """
        flight_number: NotRequired[str]
        """
        The flight number.
        """
        service_class: NotRequired[str]
        """
        The flight's service class.
        """
        stopover_allowed: NotRequired[bool]
        """
        Whether a stopover is allowed on this flight.
        """

    class CaptureParamsPurchaseDetailsFuel(TypedDict):
        industry_product_code: NotRequired[str]
        """
        [Conexxus Payment System Product Code](https://www.conexxus.org/conexxus-payment-system-product-codes) identifying the primary fuel product purchased.
        """
        quantity_decimal: NotRequired[str]
        """
        The quantity of `unit`s of fuel that was dispensed, represented as a decimal string with at most 12 decimal places.
        """
        type: NotRequired[
            Literal[
                "diesel",
                "other",
                "unleaded_plus",
                "unleaded_regular",
                "unleaded_super",
            ]
        ]
        """
        The type of fuel that was purchased. One of `diesel`, `unleaded_plus`, `unleaded_regular`, `unleaded_super`, or `other`.
        """
        unit: NotRequired[
            Literal[
                "charging_minute",
                "imperial_gallon",
                "kilogram",
                "kilowatt_hour",
                "liter",
                "other",
                "pound",
                "us_gallon",
            ]
        ]
        """
        The units for `quantity_decimal`. One of `charging_minute`, `imperial_gallon`, `kilogram`, `kilowatt_hour`, `liter`, `pound`, `us_gallon`, or `other`.
        """
        unit_cost_decimal: NotRequired[str]
        """
        The cost in cents per each unit of fuel, represented as a decimal string with at most 12 decimal places.
        """

    class CaptureParamsPurchaseDetailsLodging(TypedDict):
        check_in_at: NotRequired[int]
        """
        The time of checking into the lodging.
        """
        nights: NotRequired[int]
        """
        The number of nights stayed at the lodging.
        """

    class CaptureParamsPurchaseDetailsReceipt(TypedDict):
        description: NotRequired[str]
        quantity: NotRequired[str]
        total: NotRequired[int]
        unit_cost: NotRequired[int]

    class CreateParams(TypedDict):
        amount: int
        """
        The total amount to attempt to authorize. This amount is in the provided currency, or defaults to the card's currency, and in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
        """
        amount_details: NotRequired[
            "AuthorizationService.CreateParamsAmountDetails"
        ]
        """
        Detailed breakdown of amount components. These amounts are denominated in `currency` and in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
        """
        authorization_method: NotRequired[
            Literal["chip", "contactless", "keyed_in", "online", "swipe"]
        ]
        """
        How the card details were provided. Defaults to online.
        """
        card: str
        """
        Card associated with this authorization.
        """
        currency: NotRequired[str]
        """
        The currency of the authorization. If not provided, defaults to the currency of the card. Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
        """
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        fleet: NotRequired["AuthorizationService.CreateParamsFleet"]
        """
        Fleet-specific information for authorizations using Fleet cards.
        """
        fuel: NotRequired["AuthorizationService.CreateParamsFuel"]
        """
        Information about fuel that was purchased with this transaction.
        """
        is_amount_controllable: NotRequired[bool]
        """
        If set `true`, you may provide [amount](https://stripe.com/docs/api/issuing/authorizations/approve#approve_issuing_authorization-amount) to control how much to hold for the authorization.
        """
        merchant_data: NotRequired[
            "AuthorizationService.CreateParamsMerchantData"
        ]
        """
        Details about the seller (grocery store, e-commerce website, etc.) where the card authorization happened.
        """
        network_data: NotRequired[
            "AuthorizationService.CreateParamsNetworkData"
        ]
        """
        Details about the authorization, such as identifiers, set by the card network.
        """
        verification_data: NotRequired[
            "AuthorizationService.CreateParamsVerificationData"
        ]
        """
        Verifications that Stripe performed on information that the cardholder provided to the merchant.
        """
        wallet: NotRequired[Literal["apple_pay", "google_pay", "samsung_pay"]]
        """
        The digital wallet used for this transaction. One of `apple_pay`, `google_pay`, or `samsung_pay`. Will populate as `null` when no digital wallet was utilized.
        """

    class CreateParamsAmountDetails(TypedDict):
        atm_fee: NotRequired[int]
        """
        The ATM withdrawal fee.
        """
        cashback_amount: NotRequired[int]
        """
        The amount of cash requested by the cardholder.
        """

    class CreateParamsFleet(TypedDict):
        cardholder_prompt_data: NotRequired[
            "AuthorizationService.CreateParamsFleetCardholderPromptData"
        ]
        """
        Answers to prompts presented to the cardholder at the point of sale. Prompted fields vary depending on the configuration of your physical fleet cards. Typical points of sale support only numeric entry.
        """
        purchase_type: NotRequired[
            Literal[
                "fuel_and_non_fuel_purchase",
                "fuel_purchase",
                "non_fuel_purchase",
            ]
        ]
        """
        The type of purchase. One of `fuel_purchase`, `non_fuel_purchase`, or `fuel_and_non_fuel_purchase`.
        """
        reported_breakdown: NotRequired[
            "AuthorizationService.CreateParamsFleetReportedBreakdown"
        ]
        """
        More information about the total amount. This information is not guaranteed to be accurate as some merchants may provide unreliable data.
        """
        service_type: NotRequired[
            Literal["full_service", "non_fuel_transaction", "self_service"]
        ]
        """
        The type of fuel service. One of `non_fuel_transaction`, `full_service`, or `self_service`.
        """

    class CreateParamsFleetCardholderPromptData(TypedDict):
        driver_id: NotRequired[str]
        """
        Driver ID.
        """
        odometer: NotRequired[int]
        """
        Odometer reading.
        """
        unspecified_id: NotRequired[str]
        """
        An alphanumeric ID. This field is used when a vehicle ID, driver ID, or generic ID is entered by the cardholder, but the merchant or card network did not specify the prompt type.
        """
        user_id: NotRequired[str]
        """
        User ID.
        """
        vehicle_number: NotRequired[str]
        """
        Vehicle number.
        """

    class CreateParamsFleetReportedBreakdown(TypedDict):
        fuel: NotRequired[
            "AuthorizationService.CreateParamsFleetReportedBreakdownFuel"
        ]
        """
        Breakdown of fuel portion of the purchase.
        """
        non_fuel: NotRequired[
            "AuthorizationService.CreateParamsFleetReportedBreakdownNonFuel"
        ]
        """
        Breakdown of non-fuel portion of the purchase.
        """
        tax: NotRequired[
            "AuthorizationService.CreateParamsFleetReportedBreakdownTax"
        ]
        """
        Information about tax included in this transaction.
        """

    class CreateParamsFleetReportedBreakdownFuel(TypedDict):
        gross_amount_decimal: NotRequired[str]
        """
        Gross fuel amount that should equal Fuel Volume multipled by Fuel Unit Cost, inclusive of taxes.
        """

    class CreateParamsFleetReportedBreakdownNonFuel(TypedDict):
        gross_amount_decimal: NotRequired[str]
        """
        Gross non-fuel amount that should equal the sum of the line items, inclusive of taxes.
        """

    class CreateParamsFleetReportedBreakdownTax(TypedDict):
        local_amount_decimal: NotRequired[str]
        """
        Amount of state or provincial Sales Tax included in the transaction amount. Null if not reported by merchant or not subject to tax.
        """
        national_amount_decimal: NotRequired[str]
        """
        Amount of national Sales Tax or VAT included in the transaction amount. Null if not reported by merchant or not subject to tax.
        """

    class CreateParamsFuel(TypedDict):
        industry_product_code: NotRequired[str]
        """
        [Conexxus Payment System Product Code](https://www.conexxus.org/conexxus-payment-system-product-codes) identifying the primary fuel product purchased.
        """
        quantity_decimal: NotRequired[str]
        """
        The quantity of `unit`s of fuel that was dispensed, represented as a decimal string with at most 12 decimal places.
        """
        type: NotRequired[
            Literal[
                "diesel",
                "other",
                "unleaded_plus",
                "unleaded_regular",
                "unleaded_super",
            ]
        ]
        """
        The type of fuel that was purchased. One of `diesel`, `unleaded_plus`, `unleaded_regular`, `unleaded_super`, or `other`.
        """
        unit: NotRequired[
            Literal[
                "charging_minute",
                "imperial_gallon",
                "kilogram",
                "kilowatt_hour",
                "liter",
                "other",
                "pound",
                "us_gallon",
            ]
        ]
        """
        The units for `quantity_decimal`. One of `charging_minute`, `imperial_gallon`, `kilogram`, `kilowatt_hour`, `liter`, `pound`, `us_gallon`, or `other`.
        """
        unit_cost_decimal: NotRequired[str]
        """
        The cost in cents per each unit of fuel, represented as a decimal string with at most 12 decimal places.
        """

    class CreateParamsMerchantData(TypedDict):
        category: NotRequired[
            Literal[
                "ac_refrigeration_repair",
                "accounting_bookkeeping_services",
                "advertising_services",
                "agricultural_cooperative",
                "airlines_air_carriers",
                "airports_flying_fields",
                "ambulance_services",
                "amusement_parks_carnivals",
                "antique_reproductions",
                "antique_shops",
                "aquariums",
                "architectural_surveying_services",
                "art_dealers_and_galleries",
                "artists_supply_and_craft_shops",
                "auto_and_home_supply_stores",
                "auto_body_repair_shops",
                "auto_paint_shops",
                "auto_service_shops",
                "automated_cash_disburse",
                "automated_fuel_dispensers",
                "automobile_associations",
                "automotive_parts_and_accessories_stores",
                "automotive_tire_stores",
                "bail_and_bond_payments",
                "bakeries",
                "bands_orchestras",
                "barber_and_beauty_shops",
                "betting_casino_gambling",
                "bicycle_shops",
                "billiard_pool_establishments",
                "boat_dealers",
                "boat_rentals_and_leases",
                "book_stores",
                "books_periodicals_and_newspapers",
                "bowling_alleys",
                "bus_lines",
                "business_secretarial_schools",
                "buying_shopping_services",
                "cable_satellite_and_other_pay_television_and_radio",
                "camera_and_photographic_supply_stores",
                "candy_nut_and_confectionery_stores",
                "car_and_truck_dealers_new_used",
                "car_and_truck_dealers_used_only",
                "car_rental_agencies",
                "car_washes",
                "carpentry_services",
                "carpet_upholstery_cleaning",
                "caterers",
                "charitable_and_social_service_organizations_fundraising",
                "chemicals_and_allied_products",
                "child_care_services",
                "childrens_and_infants_wear_stores",
                "chiropodists_podiatrists",
                "chiropractors",
                "cigar_stores_and_stands",
                "civic_social_fraternal_associations",
                "cleaning_and_maintenance",
                "clothing_rental",
                "colleges_universities",
                "commercial_equipment",
                "commercial_footwear",
                "commercial_photography_art_and_graphics",
                "commuter_transport_and_ferries",
                "computer_network_services",
                "computer_programming",
                "computer_repair",
                "computer_software_stores",
                "computers_peripherals_and_software",
                "concrete_work_services",
                "construction_materials",
                "consulting_public_relations",
                "correspondence_schools",
                "cosmetic_stores",
                "counseling_services",
                "country_clubs",
                "courier_services",
                "court_costs",
                "credit_reporting_agencies",
                "cruise_lines",
                "dairy_products_stores",
                "dance_hall_studios_schools",
                "dating_escort_services",
                "dentists_orthodontists",
                "department_stores",
                "detective_agencies",
                "digital_goods_applications",
                "digital_goods_games",
                "digital_goods_large_volume",
                "digital_goods_media",
                "direct_marketing_catalog_merchant",
                "direct_marketing_combination_catalog_and_retail_merchant",
                "direct_marketing_inbound_telemarketing",
                "direct_marketing_insurance_services",
                "direct_marketing_other",
                "direct_marketing_outbound_telemarketing",
                "direct_marketing_subscription",
                "direct_marketing_travel",
                "discount_stores",
                "doctors",
                "door_to_door_sales",
                "drapery_window_covering_and_upholstery_stores",
                "drinking_places",
                "drug_stores_and_pharmacies",
                "drugs_drug_proprietaries_and_druggist_sundries",
                "dry_cleaners",
                "durable_goods",
                "duty_free_stores",
                "eating_places_restaurants",
                "educational_services",
                "electric_razor_stores",
                "electric_vehicle_charging",
                "electrical_parts_and_equipment",
                "electrical_services",
                "electronics_repair_shops",
                "electronics_stores",
                "elementary_secondary_schools",
                "emergency_services_gcas_visa_use_only",
                "employment_temp_agencies",
                "equipment_rental",
                "exterminating_services",
                "family_clothing_stores",
                "fast_food_restaurants",
                "financial_institutions",
                "fines_government_administrative_entities",
                "fireplace_fireplace_screens_and_accessories_stores",
                "floor_covering_stores",
                "florists",
                "florists_supplies_nursery_stock_and_flowers",
                "freezer_and_locker_meat_provisioners",
                "fuel_dealers_non_automotive",
                "funeral_services_crematories",
                "furniture_home_furnishings_and_equipment_stores_except_appliances",
                "furniture_repair_refinishing",
                "furriers_and_fur_shops",
                "general_services",
                "gift_card_novelty_and_souvenir_shops",
                "glass_paint_and_wallpaper_stores",
                "glassware_crystal_stores",
                "golf_courses_public",
                "government_licensed_horse_dog_racing_us_region_only",
                "government_licensed_online_casions_online_gambling_us_region_only",
                "government_owned_lotteries_non_us_region",
                "government_owned_lotteries_us_region_only",
                "government_services",
                "grocery_stores_supermarkets",
                "hardware_equipment_and_supplies",
                "hardware_stores",
                "health_and_beauty_spas",
                "hearing_aids_sales_and_supplies",
                "heating_plumbing_a_c",
                "hobby_toy_and_game_shops",
                "home_supply_warehouse_stores",
                "hospitals",
                "hotels_motels_and_resorts",
                "household_appliance_stores",
                "industrial_supplies",
                "information_retrieval_services",
                "insurance_default",
                "insurance_underwriting_premiums",
                "intra_company_purchases",
                "jewelry_stores_watches_clocks_and_silverware_stores",
                "landscaping_services",
                "laundries",
                "laundry_cleaning_services",
                "legal_services_attorneys",
                "luggage_and_leather_goods_stores",
                "lumber_building_materials_stores",
                "manual_cash_disburse",
                "marinas_service_and_supplies",
                "marketplaces",
                "masonry_stonework_and_plaster",
                "massage_parlors",
                "medical_and_dental_labs",
                "medical_dental_ophthalmic_and_hospital_equipment_and_supplies",
                "medical_services",
                "membership_organizations",
                "mens_and_boys_clothing_and_accessories_stores",
                "mens_womens_clothing_stores",
                "metal_service_centers",
                "miscellaneous_apparel_and_accessory_shops",
                "miscellaneous_auto_dealers",
                "miscellaneous_business_services",
                "miscellaneous_food_stores",
                "miscellaneous_general_merchandise",
                "miscellaneous_general_services",
                "miscellaneous_home_furnishing_specialty_stores",
                "miscellaneous_publishing_and_printing",
                "miscellaneous_recreation_services",
                "miscellaneous_repair_shops",
                "miscellaneous_specialty_retail",
                "mobile_home_dealers",
                "motion_picture_theaters",
                "motor_freight_carriers_and_trucking",
                "motor_homes_dealers",
                "motor_vehicle_supplies_and_new_parts",
                "motorcycle_shops_and_dealers",
                "motorcycle_shops_dealers",
                "music_stores_musical_instruments_pianos_and_sheet_music",
                "news_dealers_and_newsstands",
                "non_fi_money_orders",
                "non_fi_stored_value_card_purchase_load",
                "nondurable_goods",
                "nurseries_lawn_and_garden_supply_stores",
                "nursing_personal_care",
                "office_and_commercial_furniture",
                "opticians_eyeglasses",
                "optometrists_ophthalmologist",
                "orthopedic_goods_prosthetic_devices",
                "osteopaths",
                "package_stores_beer_wine_and_liquor",
                "paints_varnishes_and_supplies",
                "parking_lots_garages",
                "passenger_railways",
                "pawn_shops",
                "pet_shops_pet_food_and_supplies",
                "petroleum_and_petroleum_products",
                "photo_developing",
                "photographic_photocopy_microfilm_equipment_and_supplies",
                "photographic_studios",
                "picture_video_production",
                "piece_goods_notions_and_other_dry_goods",
                "plumbing_heating_equipment_and_supplies",
                "political_organizations",
                "postal_services_government_only",
                "precious_stones_and_metals_watches_and_jewelry",
                "professional_services",
                "public_warehousing_and_storage",
                "quick_copy_repro_and_blueprint",
                "railroads",
                "real_estate_agents_and_managers_rentals",
                "record_stores",
                "recreational_vehicle_rentals",
                "religious_goods_stores",
                "religious_organizations",
                "roofing_siding_sheet_metal",
                "secretarial_support_services",
                "security_brokers_dealers",
                "service_stations",
                "sewing_needlework_fabric_and_piece_goods_stores",
                "shoe_repair_hat_cleaning",
                "shoe_stores",
                "small_appliance_repair",
                "snowmobile_dealers",
                "special_trade_services",
                "specialty_cleaning",
                "sporting_goods_stores",
                "sporting_recreation_camps",
                "sports_and_riding_apparel_stores",
                "sports_clubs_fields",
                "stamp_and_coin_stores",
                "stationary_office_supplies_printing_and_writing_paper",
                "stationery_stores_office_and_school_supply_stores",
                "swimming_pools_sales",
                "t_ui_travel_germany",
                "tailors_alterations",
                "tax_payments_government_agencies",
                "tax_preparation_services",
                "taxicabs_limousines",
                "telecommunication_equipment_and_telephone_sales",
                "telecommunication_services",
                "telegraph_services",
                "tent_and_awning_shops",
                "testing_laboratories",
                "theatrical_ticket_agencies",
                "timeshares",
                "tire_retreading_and_repair",
                "tolls_bridge_fees",
                "tourist_attractions_and_exhibits",
                "towing_services",
                "trailer_parks_campgrounds",
                "transportation_services",
                "travel_agencies_tour_operators",
                "truck_stop_iteration",
                "truck_utility_trailer_rentals",
                "typesetting_plate_making_and_related_services",
                "typewriter_stores",
                "u_s_federal_government_agencies_or_departments",
                "uniforms_commercial_clothing",
                "used_merchandise_and_secondhand_stores",
                "utilities",
                "variety_stores",
                "veterinary_services",
                "video_amusement_game_supplies",
                "video_game_arcades",
                "video_tape_rental_stores",
                "vocational_trade_schools",
                "watch_jewelry_repair",
                "welding_repair",
                "wholesale_clubs",
                "wig_and_toupee_stores",
                "wires_money_orders",
                "womens_accessory_and_specialty_shops",
                "womens_ready_to_wear_stores",
                "wrecking_and_salvage_yards",
            ]
        ]
        """
        A categorization of the seller's type of business. See our [merchant categories guide](https://stripe.com/docs/issuing/merchant-categories) for a list of possible values.
        """
        city: NotRequired[str]
        """
        City where the seller is located
        """
        country: NotRequired[str]
        """
        Country where the seller is located
        """
        name: NotRequired[str]
        """
        Name of the seller
        """
        network_id: NotRequired[str]
        """
        Identifier assigned to the seller by the card network. Different card networks may assign different network_id fields to the same merchant.
        """
        postal_code: NotRequired[str]
        """
        Postal code where the seller is located
        """
        state: NotRequired[str]
        """
        State where the seller is located
        """
        terminal_id: NotRequired[str]
        """
        An ID assigned by the seller to the location of the sale.
        """
        url: NotRequired[str]
        """
        URL provided by the merchant on a 3DS request
        """

    class CreateParamsNetworkData(TypedDict):
        acquiring_institution_id: NotRequired[str]
        """
        Identifier assigned to the acquirer by the card network.
        """

    class CreateParamsVerificationData(TypedDict):
        address_line1_check: NotRequired[
            Literal["match", "mismatch", "not_provided"]
        ]
        """
        Whether the cardholder provided an address first line and if it matched the cardholder's `billing.address.line1`.
        """
        address_postal_code_check: NotRequired[
            Literal["match", "mismatch", "not_provided"]
        ]
        """
        Whether the cardholder provided a postal code and if it matched the cardholder's `billing.address.postal_code`.
        """
        authentication_exemption: NotRequired[
            "AuthorizationService.CreateParamsVerificationDataAuthenticationExemption"
        ]
        """
        The exemption applied to this authorization.
        """
        cvc_check: NotRequired[Literal["match", "mismatch", "not_provided"]]
        """
        Whether the cardholder provided a CVC and if it matched Stripe's record.
        """
        expiry_check: NotRequired[Literal["match", "mismatch", "not_provided"]]
        """
        Whether the cardholder provided an expiry date and if it matched Stripe's record.
        """
        three_d_secure: NotRequired[
            "AuthorizationService.CreateParamsVerificationDataThreeDSecure"
        ]
        """
        3D Secure details.
        """

    class CreateParamsVerificationDataAuthenticationExemption(TypedDict):
        claimed_by: Literal["acquirer", "issuer"]
        """
        The entity that requested the exemption, either the acquiring merchant or the Issuing user.
        """
        type: Literal[
            "low_value_transaction", "transaction_risk_analysis", "unknown"
        ]
        """
        The specific exemption claimed for this authorization.
        """

    class CreateParamsVerificationDataThreeDSecure(TypedDict):
        result: Literal[
            "attempt_acknowledged", "authenticated", "failed", "required"
        ]
        """
        The outcome of the 3D Secure authentication request.
        """

    class ExpireParams(TypedDict):
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """

    class FinalizeAmountParams(TypedDict):
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        final_amount: int
        """
        The final authorization amount that will be captured by the merchant. This amount is in the authorization currency and in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
        """
        fleet: NotRequired["AuthorizationService.FinalizeAmountParamsFleet"]
        """
        Fleet-specific information for authorizations using Fleet cards.
        """
        fuel: NotRequired["AuthorizationService.FinalizeAmountParamsFuel"]
        """
        Information about fuel that was purchased with this transaction.
        """

    class FinalizeAmountParamsFleet(TypedDict):
        cardholder_prompt_data: NotRequired[
            "AuthorizationService.FinalizeAmountParamsFleetCardholderPromptData"
        ]
        """
        Answers to prompts presented to the cardholder at the point of sale. Prompted fields vary depending on the configuration of your physical fleet cards. Typical points of sale support only numeric entry.
        """
        purchase_type: NotRequired[
            Literal[
                "fuel_and_non_fuel_purchase",
                "fuel_purchase",
                "non_fuel_purchase",
            ]
        ]
        """
        The type of purchase. One of `fuel_purchase`, `non_fuel_purchase`, or `fuel_and_non_fuel_purchase`.
        """
        reported_breakdown: NotRequired[
            "AuthorizationService.FinalizeAmountParamsFleetReportedBreakdown"
        ]
        """
        More information about the total amount. This information is not guaranteed to be accurate as some merchants may provide unreliable data.
        """
        service_type: NotRequired[
            Literal["full_service", "non_fuel_transaction", "self_service"]
        ]
        """
        The type of fuel service. One of `non_fuel_transaction`, `full_service`, or `self_service`.
        """

    class FinalizeAmountParamsFleetCardholderPromptData(TypedDict):
        driver_id: NotRequired[str]
        """
        Driver ID.
        """
        odometer: NotRequired[int]
        """
        Odometer reading.
        """
        unspecified_id: NotRequired[str]
        """
        An alphanumeric ID. This field is used when a vehicle ID, driver ID, or generic ID is entered by the cardholder, but the merchant or card network did not specify the prompt type.
        """
        user_id: NotRequired[str]
        """
        User ID.
        """
        vehicle_number: NotRequired[str]
        """
        Vehicle number.
        """

    class FinalizeAmountParamsFleetReportedBreakdown(TypedDict):
        fuel: NotRequired[
            "AuthorizationService.FinalizeAmountParamsFleetReportedBreakdownFuel"
        ]
        """
        Breakdown of fuel portion of the purchase.
        """
        non_fuel: NotRequired[
            "AuthorizationService.FinalizeAmountParamsFleetReportedBreakdownNonFuel"
        ]
        """
        Breakdown of non-fuel portion of the purchase.
        """
        tax: NotRequired[
            "AuthorizationService.FinalizeAmountParamsFleetReportedBreakdownTax"
        ]
        """
        Information about tax included in this transaction.
        """

    class FinalizeAmountParamsFleetReportedBreakdownFuel(TypedDict):
        gross_amount_decimal: NotRequired[str]
        """
        Gross fuel amount that should equal Fuel Volume multipled by Fuel Unit Cost, inclusive of taxes.
        """

    class FinalizeAmountParamsFleetReportedBreakdownNonFuel(TypedDict):
        gross_amount_decimal: NotRequired[str]
        """
        Gross non-fuel amount that should equal the sum of the line items, inclusive of taxes.
        """

    class FinalizeAmountParamsFleetReportedBreakdownTax(TypedDict):
        local_amount_decimal: NotRequired[str]
        """
        Amount of state or provincial Sales Tax included in the transaction amount. Null if not reported by merchant or not subject to tax.
        """
        national_amount_decimal: NotRequired[str]
        """
        Amount of national Sales Tax or VAT included in the transaction amount. Null if not reported by merchant or not subject to tax.
        """

    class FinalizeAmountParamsFuel(TypedDict):
        industry_product_code: NotRequired[str]
        """
        [Conexxus Payment System Product Code](https://www.conexxus.org/conexxus-payment-system-product-codes) identifying the primary fuel product purchased.
        """
        quantity_decimal: NotRequired[str]
        """
        The quantity of `unit`s of fuel that was dispensed, represented as a decimal string with at most 12 decimal places.
        """
        type: NotRequired[
            Literal[
                "diesel",
                "other",
                "unleaded_plus",
                "unleaded_regular",
                "unleaded_super",
            ]
        ]
        """
        The type of fuel that was purchased. One of `diesel`, `unleaded_plus`, `unleaded_regular`, `unleaded_super`, or `other`.
        """
        unit: NotRequired[
            Literal[
                "charging_minute",
                "imperial_gallon",
                "kilogram",
                "kilowatt_hour",
                "liter",
                "other",
                "pound",
                "us_gallon",
            ]
        ]
        """
        The units for `quantity_decimal`. One of `charging_minute`, `imperial_gallon`, `kilogram`, `kilowatt_hour`, `liter`, `pound`, `us_gallon`, or `other`.
        """
        unit_cost_decimal: NotRequired[str]
        """
        The cost in cents per each unit of fuel, represented as a decimal string with at most 12 decimal places.
        """

    class IncrementParams(TypedDict):
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        increment_amount: int
        """
        The amount to increment the authorization by. This amount is in the authorization currency and in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
        """
        is_amount_controllable: NotRequired[bool]
        """
        If set `true`, you may provide [amount](https://stripe.com/docs/api/issuing/authorizations/approve#approve_issuing_authorization-amount) to control how much to hold for the authorization.
        """

    class ReverseParams(TypedDict):
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        reverse_amount: NotRequired[int]
        """
        The amount to reverse from the authorization. If not provided, the full amount of the authorization will be reversed. This amount is in the authorization currency and in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
        """

    def create(
        self,
        params: "AuthorizationService.CreateParams",
        options: RequestOptions = {},
    ) -> Authorization:
        """
        Create a test-mode authorization.
        """
        return cast(
            Authorization,
            self._request(
                "post",
                "/v1/test_helpers/issuing/authorizations",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "AuthorizationService.CreateParams",
        options: RequestOptions = {},
    ) -> Authorization:
        """
        Create a test-mode authorization.
        """
        return cast(
            Authorization,
            await self._request_async(
                "post",
                "/v1/test_helpers/issuing/authorizations",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def capture(
        self,
        authorization: str,
        params: "AuthorizationService.CaptureParams" = {},
        options: RequestOptions = {},
    ) -> Authorization:
        """
        Capture a test-mode authorization.
        """
        return cast(
            Authorization,
            self._request(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/capture".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def capture_async(
        self,
        authorization: str,
        params: "AuthorizationService.CaptureParams" = {},
        options: RequestOptions = {},
    ) -> Authorization:
        """
        Capture a test-mode authorization.
        """
        return cast(
            Authorization,
            await self._request_async(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/capture".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def expire(
        self,
        authorization: str,
        params: "AuthorizationService.ExpireParams" = {},
        options: RequestOptions = {},
    ) -> Authorization:
        """
        Expire a test-mode Authorization.
        """
        return cast(
            Authorization,
            self._request(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/expire".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def expire_async(
        self,
        authorization: str,
        params: "AuthorizationService.ExpireParams" = {},
        options: RequestOptions = {},
    ) -> Authorization:
        """
        Expire a test-mode Authorization.
        """
        return cast(
            Authorization,
            await self._request_async(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/expire".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def finalize_amount(
        self,
        authorization: str,
        params: "AuthorizationService.FinalizeAmountParams",
        options: RequestOptions = {},
    ) -> Authorization:
        """
        Finalize the amount on an Authorization prior to capture, when the initial authorization was for an estimated amount.
        """
        return cast(
            Authorization,
            self._request(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/finalize_amount".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def finalize_amount_async(
        self,
        authorization: str,
        params: "AuthorizationService.FinalizeAmountParams",
        options: RequestOptions = {},
    ) -> Authorization:
        """
        Finalize the amount on an Authorization prior to capture, when the initial authorization was for an estimated amount.
        """
        return cast(
            Authorization,
            await self._request_async(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/finalize_amount".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def increment(
        self,
        authorization: str,
        params: "AuthorizationService.IncrementParams",
        options: RequestOptions = {},
    ) -> Authorization:
        """
        Increment a test-mode Authorization.
        """
        return cast(
            Authorization,
            self._request(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/increment".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def increment_async(
        self,
        authorization: str,
        params: "AuthorizationService.IncrementParams",
        options: RequestOptions = {},
    ) -> Authorization:
        """
        Increment a test-mode Authorization.
        """
        return cast(
            Authorization,
            await self._request_async(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/increment".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def reverse(
        self,
        authorization: str,
        params: "AuthorizationService.ReverseParams" = {},
        options: RequestOptions = {},
    ) -> Authorization:
        """
        Reverse a test-mode Authorization.
        """
        return cast(
            Authorization,
            self._request(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/reverse".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def reverse_async(
        self,
        authorization: str,
        params: "AuthorizationService.ReverseParams" = {},
        options: RequestOptions = {},
    ) -> Authorization:
        """
        Reverse a test-mode Authorization.
        """
        return cast(
            Authorization,
            await self._request_async(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/reverse".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe.gift_cards._card_service import CardService
from stripe.gift_cards._transaction_service import TransactionService


class GiftCardsService(StripeService):
    def __init__(self, requestor):
        super().__init__(requestor)
        self.cards = CardService(self._requestor)
        self.transactions = TransactionService(self._requestor)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe.issuing._authorization_service import AuthorizationService
from stripe.issuing._card_service import CardService
from stripe.issuing._cardholder_service import CardholderService
from stripe.issuing._credit_underwriting_record_service import (
    CreditUnderwritingRecordService,
)
from stripe.issuing._dispute_service import DisputeService
from stripe.issuing._dispute_settlement_detail_service import (
    DisputeSettlementDetailService,
)
from stripe.issuing._personalization_design_service import (
    PersonalizationDesignService,
)
from stripe.issuing._physical_bundle_service import PhysicalBundleService
from stripe.issuing._token_service import TokenService
from stripe.issuing._transaction_service import TransactionService


class IssuingService(StripeService):
    def __init__(self, requestor):
        super().__init__(requestor)
        self.authorizations = AuthorizationService(self._requestor)
        self.cards = CardService(self._requestor)
        self.cardholders = CardholderService(self._requestor)
        self.credit_underwriting_records = CreditUnderwritingRecordService(
            self._requestor,
        )
        self.disputes = DisputeService(self._requestor)
        self.dispute_settlement_details = DisputeSettlementDetailService(
            self._requestor,
        )
        self.personalization_designs = PersonalizationDesignService(
            self._requestor,
        )
        self.physical_bundles = PhysicalBundleService(self._requestor)
        self.tokens = TokenService(self._requestor)
        self.transactions = TransactionService(self._requestor)



from stripe import raw_request, raw_request_async


class Preview(object):
    def _get_default_opts(self, params):
        if "api_mode" not in params:
            params["api_mode"] = "preview"
        return params

    def post(self, url, **params):
        return raw_request("post", url, **self._get_default_opts(params))

    def get(self, url, **params):
        return raw_request("get", url, **self._get_default_opts(params))

    def delete(self, url, **params):
        return raw_request("delete", url, **self._get_default_opts(params))

    def post_async(self, url, **params):
        return raw_request_async("post", url, **self._get_default_opts(params))

    def get_async(self, url, **params):
        return raw_request_async("get", url, **self._get_default_opts(params))

    def delete_async(self, url, **params):
        return raw_request_async(
            "delete", url, **self._get_default_opts(params)
        )


preview = Preview()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe.test_helpers.issuing._authorization_service import (
    AuthorizationService as AuthorizationService,
)
from stripe.test_helpers.issuing._card_service import (
    CardService as CardService,
)
from stripe.test_helpers.issuing._personalization_design_service import (
    PersonalizationDesignService as PersonalizationDesignService,
)
from stripe.test_helpers.issuing._transaction_service import (
    TransactionService as TransactionService,
)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._request_options import RequestOptions
from stripe._stripe_object import StripeObject
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, Dict, List, Optional, cast, overload
from typing_extensions import Literal, NotRequired, TypedDict, Unpack


class CreditUnderwritingRecord(
    ListableAPIResource["CreditUnderwritingRecord"]
):
    """
    Every time an applicant submits an application for a Charge Card product your platform offers, or every time your platform takes a proactive credit decision on an existing account, you must record the decision by creating a new `CreditUnderwritingRecord` object on a connected account.

    [Follow the guide](https://stripe.com/docs/issuing/credit/report-credit-decisions-and-manage-aans) to learn about your requirements as a platform.
    """

    OBJECT_NAME: ClassVar[Literal["issuing.credit_underwriting_record"]] = (
        "issuing.credit_underwriting_record"
    )

    class Application(StripeObject):
        application_method: Literal["in_person", "mail", "online", "phone"]
        """
        The channel through which the applicant has submitted their application.
        """
        purpose: Literal["credit_limit_increase", "credit_line_opening"]
        """
        Scope of demand made by the applicant.
        """
        submitted_at: int
        """
        Date when the applicant submitted their application.
        """

    class CreditUser(StripeObject):
        email: str
        """
        Email of the applicant or accountholder.
        """
        name: str
        """
        Full name of the company or person.
        """

    class Decision(StripeObject):
        class ApplicationRejected(StripeObject):
            reason_other_explanation: Optional[str]
            """
            Details about the `reasons.other` when present.
            """
            reasons: List[
                Literal[
                    "applicant_is_not_beneficial_owner",
                    "applicant_too_young",
                    "application_is_not_beneficial_owner",
                    "bankruptcy",
                    "business_size_too_small",
                    "current_account_tier_ineligible",
                    "customer_already_exists",
                    "customer_requested_account_closure",
                    "debt_to_cash_balance_ratio_too_high",
                    "debt_to_equity_ratio_too_high",
                    "delinquent_credit_obligations",
                    "dispute_rate_too_high",
                    "duration_of_residence",
                    "excessive_income_or_revenue_obligations",
                    "expenses_to_cash_balance_ratio_too_high",
                    "foreclosure_or_repossession",
                    "frozen_file_at_credit_bureau",
                    "garnishment_or_attachment",
                    "government_loan_program_criteria",
                    "high_concentration_of_clients",
                    "high_risk_industry",
                    "incomplete_application",
                    "inconsistent_monthly_revenues",
                    "insufficient_account_history_with_platform",
                    "insufficient_bank_account_history",
                    "insufficient_cash_balance",
                    "insufficient_cash_flow",
                    "insufficient_collateral",
                    "insufficient_credit_experience",
                    "insufficient_deposits",
                    "insufficient_income",
                    "insufficient_margin_ratio",
                    "insufficient_operating_profit",
                    "insufficient_period_in_operation",
                    "insufficient_reserves",
                    "insufficient_revenue",
                    "insufficient_social_media_performance",
                    "insufficient_time_in_network",
                    "insufficient_trade_credit_insurance",
                    "invalid_business_license",
                    "lacking_cash_account",
                    "late_payment_history_reported_to_bureau",
                    "lien_collection_action_or_judgement",
                    "negative_public_information",
                    "no_credit_file",
                    "other",
                    "outside_supported_country",
                    "outside_supported_state",
                    "poor_payment_history_with_platform",
                    "prior_or_current_legal_action",
                    "prohibited_industry",
                    "rate_of_cash_balance_fluctuation_too_high",
                    "recent_inquiries_on_business_credit_report",
                    "removal_of_bank_account_connection",
                    "revenue_discrepancy",
                    "runway_too_short",
                    "suspected_fraud",
                    "too_many_non_sufficient_funds_or_overdrafts",
                    "unable_to_verify_address",
                    "unable_to_verify_identity",
                    "unable_to_verify_income_or_revenue",
                    "unprofitable",
                    "unsupportable_business_type",
                ]
            ]
            """
            List of reasons why the application was rejected up to 4 reasons, in order of importance.
            """

        class CreditLimitApproved(StripeObject):
            amount: int
            """
            Credit amount approved. An approved credit limit is required before you can set a amount in the [CreditPolicy API](https://stripe.com/docs/api/issuing/credit_policy).
            """
            currency: str
            """
            Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
            """

        class CreditLimitDecreased(StripeObject):
            amount: int
            """
            Credit amount approved after decrease. An approved credit limit is required before you can set a amount in the [CreditPolicy API](https://stripe.com/docs/api/issuing/credit_policy).
            """
            currency: str
            """
            Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
            """
            reason_other_explanation: Optional[str]
            """
            Details about the `reasons.other` when present.
            """
            reasons: List[
                Literal[
                    "applicant_is_not_beneficial_owner",
                    "applicant_too_young",
                    "application_is_not_beneficial_owner",
                    "bankruptcy",
                    "business_size_too_small",
                    "change_in_financial_state",
                    "change_in_utilization_of_credit_line",
                    "current_account_tier_ineligible",
                    "customer_already_exists",
                    "customer_requested_account_closure",
                    "debt_to_cash_balance_ratio_too_high",
                    "debt_to_equity_ratio_too_high",
                    "decrease_in_income_to_expense_ratio",
                    "decrease_in_social_media_performance",
                    "delinquent_credit_obligations",
                    "dispute_rate_too_high",
                    "duration_of_residence",
                    "exceeds_acceptable_platform_exposure",
                    "excessive_income_or_revenue_obligations",
                    "expenses_to_cash_balance_ratio_too_high",
                    "foreclosure_or_repossession",
                    "frozen_file_at_credit_bureau",
                    "garnishment_or_attachment",
                    "government_loan_program_criteria",
                    "has_recent_credit_limit_increase",
                    "high_concentration_of_clients",
                    "high_risk_industry",
                    "incomplete_application",
                    "inconsistent_monthly_revenues",
                    "insufficient_account_history_with_platform",
                    "insufficient_bank_account_history",
                    "insufficient_cash_balance",
                    "insufficient_cash_flow",
                    "insufficient_collateral",
                    "insufficient_credit_experience",
                    "insufficient_credit_utilization",
                    "insufficient_deposits",
                    "insufficient_income",
                    "insufficient_margin_ratio",
                    "insufficient_operating_profit",
                    "insufficient_period_in_operation",
                    "insufficient_reserves",
                    "insufficient_revenue",
                    "insufficient_social_media_performance",
                    "insufficient_time_in_network",
                    "insufficient_trade_credit_insurance",
                    "insufficient_usage_as_qualified_expenses",
                    "invalid_business_license",
                    "lacking_cash_account",
                    "late_payment_history_reported_to_bureau",
                    "lien_collection_action_or_judgement",
                    "negative_public_information",
                    "no_credit_file",
                    "other",
                    "outside_supported_country",
                    "outside_supported_state",
                    "poor_payment_history_with_platform",
                    "prior_or_current_legal_action",
                    "prohibited_industry",
                    "rate_of_cash_balance_fluctuation_too_high",
                    "recent_inquiries_on_business_credit_report",
                    "removal_of_bank_account_connection",
                    "revenue_discrepancy",
                    "runway_too_short",
                    "suspected_fraud",
                    "too_many_non_sufficient_funds_or_overdrafts",
                    "unable_to_verify_address",
                    "unable_to_verify_identity",
                    "unable_to_verify_income_or_revenue",
                    "unprofitable",
                    "unsupportable_business_type",
                ]
            ]
            """
            List of reasons why the existing credit was decreased, up to 4 reasons, in order of importance.
            """

        class CreditLineClosed(StripeObject):
            reason_other_explanation: Optional[str]
            """
            Details about the `reasons.other` when present.
            """
            reasons: List[
                Literal[
                    "applicant_is_not_beneficial_owner",
                    "applicant_too_young",
                    "application_is_not_beneficial_owner",
                    "bankruptcy",
                    "business_size_too_small",
                    "change_in_financial_state",
                    "change_in_utilization_of_credit_line",
                    "current_account_tier_ineligible",
                    "customer_already_exists",
                    "customer_requested_account_closure",
                    "debt_to_cash_balance_ratio_too_high",
                    "debt_to_equity_ratio_too_high",
                    "decrease_in_income_to_expense_ratio",
                    "decrease_in_social_media_performance",
                    "delinquent_credit_obligations",
                    "dispute_rate_too_high",
                    "duration_of_residence",
                    "exceeds_acceptable_platform_exposure",
                    "excessive_income_or_revenue_obligations",
                    "expenses_to_cash_balance_ratio_too_high",
                    "foreclosure_or_repossession",
                    "frozen_file_at_credit_bureau",
                    "garnishment_or_attachment",
                    "government_loan_program_criteria",
                    "has_recent_credit_limit_increase",
                    "high_concentration_of_clients",
                    "high_risk_industry",
                    "incomplete_application",
                    "inconsistent_monthly_revenues",
                    "insufficient_account_history_with_platform",
                    "insufficient_bank_account_history",
                    "insufficient_cash_balance",
                    "insufficient_cash_flow",
                    "insufficient_collateral",
                    "insufficient_credit_experience",
                    "insufficient_credit_utilization",
                    "insufficient_deposits",
                    "insufficient_income",
                    "insufficient_margin_ratio",
                    "insufficient_operating_profit",
                    "insufficient_period_in_operation",
                    "insufficient_reserves",
                    "insufficient_revenue",
                    "insufficient_social_media_performance",
                    "insufficient_time_in_network",
                    "insufficient_trade_credit_insurance",
                    "insufficient_usage_as_qualified_expenses",
                    "invalid_business_license",
                    "lacking_cash_account",
                    "late_payment_history_reported_to_bureau",
                    "lien_collection_action_or_judgement",
                    "negative_public_information",
                    "no_credit_file",
                    "other",
                    "outside_supported_country",
                    "outside_supported_state",
                    "poor_payment_history_with_platform",
                    "prior_or_current_legal_action",
                    "prohibited_industry",
                    "rate_of_cash_balance_fluctuation_too_high",
                    "recent_inquiries_on_business_credit_report",
                    "removal_of_bank_account_connection",
                    "revenue_discrepancy",
                    "runway_too_short",
                    "suspected_fraud",
                    "too_many_non_sufficient_funds_or_overdrafts",
                    "unable_to_verify_address",
                    "unable_to_verify_identity",
                    "unable_to_verify_income_or_revenue",
                    "unprofitable",
                    "unsupportable_business_type",
                ]
            ]
            """
            List of reasons why the existing account was closed, up to 4 reasons, in order of importance.
            """

        application_rejected: Optional[ApplicationRejected]
        """
        Details about a decision application_rejected.
        """
        credit_limit_approved: Optional[CreditLimitApproved]
        """
        Details about a decision credit_limit_approved.
        """
        credit_limit_decreased: Optional[CreditLimitDecreased]
        """
        Details about a decision credit_limit_decreased.
        """
        credit_line_closed: Optional[CreditLineClosed]
        """
        Details about a decision credit_line_closed.
        """
        type: Literal[
            "additional_information_requested",
            "application_rejected",
            "credit_limit_approved",
            "credit_limit_decreased",
            "credit_line_closed",
            "no_changes",
            "withdrawn_by_applicant",
        ]
        """
        Outcome of the decision.
        """
        _inner_class_types = {
            "application_rejected": ApplicationRejected,
            "credit_limit_approved": CreditLimitApproved,
            "credit_limit_decreased": CreditLimitDecreased,
            "credit_line_closed": CreditLineClosed,
        }

    class UnderwritingException(StripeObject):
        explanation: str
        """
        Written explanation for the exception.
        """
        original_decision_type: Literal[
            "additional_information_requested",
            "application_rejected",
            "credit_limit_approved",
            "credit_limit_decreased",
            "credit_line_closed",
            "no_changes",
            "withdrawn_by_applicant",
        ]
        """
        The decision before the exception was applied.
        """

    class CorrectParams(RequestOptions):
        application: NotRequired[
            "CreditUnderwritingRecord.CorrectParamsApplication"
        ]
        """
        Details about the application submission.
        """
        credit_user: NotRequired[
            "CreditUnderwritingRecord.CorrectParamsCreditUser"
        ]
        """
        Information about the company or person applying or holding the account.
        """
        decided_at: NotRequired[int]
        """
        Date when a decision was made.
        """
        decision: NotRequired["CreditUnderwritingRecord.CorrectParamsDecision"]
        """
        Details about the decision.
        """
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        metadata: NotRequired[Dict[str, str]]
        """
        Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
        """
        regulatory_reporting_file: NotRequired[str]
        """
        File containing regulatory reporting data for the decision. Required if you are subject to this [reporting requirement](https://stripe.com/docs/issuing/credit/report-required-regulatory-data-for-credit-decisions). Optional if previously provided and no changes are needed.
        """
        underwriting_exception: NotRequired[
            "CreditUnderwritingRecord.CorrectParamsUnderwritingException"
        ]
        """
        If an exception to the usual underwriting criteria was made for this decision, details about the exception must be provided. Exceptions should only be granted in rare circumstances, in consultation with Stripe Compliance.
        """

    class CorrectParamsApplication(TypedDict):
        application_method: NotRequired[
            Literal["in_person", "mail", "online", "phone"]
        ]
        """
        The channel through which the applicant has submitted their application. Defaults to `online`.
        """
        purpose: Literal["credit_limit_increase", "credit_line_opening"]
        """
        Scope of demand made by the applicant.
        """
        submitted_at: int
        """
        Date when the applicant submitted their application.
        """

    class CorrectParamsCreditUser(TypedDict):
        email: str
        """
        Email of the applicant or accountholder.
        """
        name: str
        """
        Full name of the company or person.
        """

    class CorrectParamsDecision(TypedDict):
        application_rejected: NotRequired[
            "CreditUnderwritingRecord.CorrectParamsDecisionApplicationRejected"
        ]
        """
        Details about the application rejection.
        """
        credit_limit_approved: NotRequired[
            "CreditUnderwritingRecord.CorrectParamsDecisionCreditLimitApproved"
        ]
        """
        Details about the credit limit approved. An approved credit limit is required before you can set a `credit_limit_amount` in the [CreditPolicy API](https://stripe.com/docs/api/issuing/credit_policy/)
        """
        credit_limit_decreased: NotRequired[
            "CreditUnderwritingRecord.CorrectParamsDecisionCreditLimitDecreased"
        ]
        """
        Details about the credit limit decreased.
        """
        credit_line_closed: NotRequired[
            "CreditUnderwritingRecord.CorrectParamsDecisionCreditLineClosed"
        ]
        """
        Details about the credit line closed.
        """
        type: Literal[
            "additional_information_requested",
            "application_rejected",
            "credit_limit_approved",
            "credit_limit_decreased",
            "credit_line_closed",
            "no_changes",
            "withdrawn_by_applicant",
        ]
        """
        Outcome of the decision.
        """

    class CorrectParamsDecisionApplicationRejected(TypedDict):
        reason_other_explanation: NotRequired[str]
        """
        Details about the `reasons.other` when present.
        """
        reasons: List[
            Literal[
                "applicant_is_not_beneficial_owner",
                "applicant_too_young",
                "application_is_not_beneficial_owner",
                "bankruptcy",
                "business_size_too_small",
                "current_account_tier_ineligible",
                "customer_already_exists",
                "customer_requested_account_closure",
                "debt_to_cash_balance_ratio_too_high",
                "debt_to_equity_ratio_too_high",
                "delinquent_credit_obligations",
                "dispute_rate_too_high",
                "duration_of_residence",
                "excessive_income_or_revenue_obligations",
                "expenses_to_cash_balance_ratio_too_high",
                "foreclosure_or_repossession",
                "frozen_file_at_credit_bureau",
                "garnishment_or_attachment",
                "government_loan_program_criteria",
                "high_concentration_of_clients",
                "high_risk_industry",
                "incomplete_application",
                "inconsistent_monthly_revenues",
                "insufficient_account_history_with_platform",
                "insufficient_bank_account_history",
                "insufficient_cash_balance",
                "insufficient_cash_flow",
                "insufficient_collateral",
                "insufficient_credit_experience",
                "insufficient_deposits",
                "insufficient_income",
                "insufficient_margin_ratio",
                "insufficient_operating_profit",
                "insufficient_period_in_operation",
                "insufficient_reserves",
                "insufficient_revenue",
                "insufficient_social_media_performance",
                "insufficient_time_in_network",
                "insufficient_trade_credit_insurance",
                "invalid_business_license",
                "lacking_cash_account",
                "late_payment_history_reported_to_bureau",
                "lien_collection_action_or_judgement",
                "negative_public_information",
                "no_credit_file",
                "other",
                "outside_supported_country",
                "outside_supported_state",
                "poor_payment_history_with_platform",
                "prior_or_current_legal_action",
                "prohibited_industry",
                "rate_of_cash_balance_fluctuation_too_high",
                "recent_inquiries_on_business_credit_report",
                "removal_of_bank_account_connection",
                "revenue_discrepancy",
                "runway_too_short",
                "suspected_fraud",
                "too_many_non_sufficient_funds_or_overdrafts",
                "unable_to_verify_address",
                "unable_to_verify_identity",
                "unable_to_verify_income_or_revenue",
                "unprofitable",
                "unsupportable_business_type",
            ]
        ]
        """
        List of reasons why the application was rejected, up to 4 reasons, in order of importance.
        """

    class CorrectParamsDecisionCreditLimitApproved(TypedDict):
        amount: int
        """
        The credit approved, in the currency of the account and [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
        """
        currency: NotRequired[str]
        """
        The currency of the credit approved, will default to the Account's Issuing currency.
        """

    class CorrectParamsDecisionCreditLimitDecreased(TypedDict):
        amount: int
        """
        The credit approved, in the currency of the account and [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
        """
        currency: NotRequired[str]
        """
        The currency of the credit approved, will default to the Account's Issuing currency.
        """
        reason_other_explanation: NotRequired[str]
        """
        Details about the `reasons.other` when present.
        """
        reasons: List[
            Literal[
                "applicant_is_not_beneficial_owner",
                "applicant_too_young",
                "application_is_not_beneficial_owner",
                "bankruptcy",
                "business_size_too_small",
                "change_in_financial_state",
                "change_in_utilization_of_credit_line",
                "current_account_tier_ineligible",
                "customer_already_exists",
                "customer_requested_account_closure",
                "debt_to_cash_balance_ratio_too_high",
                "debt_to_equity_ratio_too_high",
                "decrease_in_income_to_expense_ratio",
                "decrease_in_social_media_performance",
                "delinquent_credit_obligations",
                "dispute_rate_too_high",
                "duration_of_residence",
                "exceeds_acceptable_platform_exposure",
                "excessive_income_or_revenue_obligations",
                "expenses_to_cash_balance_ratio_too_high",
                "foreclosure_or_repossession",
                "frozen_file_at_credit_bureau",
                "garnishment_or_attachment",
                "government_loan_program_criteria",
                "has_recent_credit_limit_increase",
                "high_concentration_of_clients",
                "high_risk_industry",
                "incomplete_application",
                "inconsistent_monthly_revenues",
                "insufficient_account_history_with_platform",
                "insufficient_bank_account_history",
                "insufficient_cash_balance",
                "insufficient_cash_flow",
                "insufficient_collateral",
                "insufficient_credit_experience",
                "insufficient_credit_utilization",
                "insufficient_deposits",
                "insufficient_income",
                "insufficient_margin_ratio",
                "insufficient_operating_profit",
                "insufficient_period_in_operation",
                "insufficient_reserves",
                "insufficient_revenue",
                "insufficient_social_media_performance",
                "insufficient_time_in_network",
                "insufficient_trade_credit_insurance",
                "insufficient_usage_as_qualified_expenses",
                "invalid_business_license",
                "lacking_cash_account",
                "late_payment_history_reported_to_bureau",
                "lien_collection_action_or_judgement",
                "negative_public_information",
                "no_credit_file",
                "other",
                "outside_supported_country",
                "outside_supported_state",
                "poor_payment_history_with_platform",
                "prior_or_current_legal_action",
                "prohibited_industry",
                "rate_of_cash_balance_fluctuation_too_high",
                "recent_inquiries_on_business_credit_report",
                "removal_of_bank_account_connection",
                "revenue_discrepancy",
                "runway_too_short",
                "suspected_fraud",
                "too_many_non_sufficient_funds_or_overdrafts",
                "unable_to_verify_address",
                "unable_to_verify_identity",
                "unable_to_verify_income_or_revenue",
                "unprofitable",
                "unsupportable_business_type",
            ]
        ]
        """
        List of reasons why the existing credit was decreased, up to 4 reasons, in order of importance.
        """

    class CorrectParamsDecisionCreditLineClosed(TypedDict):
        reason_other_explanation: NotRequired[str]
        """
        Details about the `reasons.other` when present.
        """
        reasons: List[
            Literal[
                "applicant_is_not_beneficial_owner",
                "applicant_too_young",
                "application_is_not_beneficial_owner",
                "bankruptcy",
                "business_size_too_small",
                "change_in_financial_state",
                "change_in_utilization_of_credit_line",
                "current_account_tier_ineligible",
                "customer_already_exists",
                "customer_requested_account_closure",
                "debt_to_cash_balance_ratio_too_high",
                "debt_to_equity_ratio_too_high",
                "decrease_in_income_to_expense_ratio",
                "decrease_in_social_media_performance",
                "delinquent_credit_obligations",
                "dispute_rate_too_high",
                "duration_of_residence",
                "exceeds_acceptable_platform_exposure",
                "excessive_income_or_revenue_obligations",
                "expenses_to_cash_balance_ratio_too_high",
                "foreclosure_or_repossession",
                "frozen_file_at_credit_bureau",
                "garnishment_or_attachment",
                "government_loan_program_criteria",
                "has_recent_credit_limit_increase",
                "high_concentration_of_clients",
                "high_risk_industry",
                "incomplete_application",
                "inconsistent_monthly_revenues",
                "insufficient_account_history_with_platform",
                "insufficient_bank_account_history",
                "insufficient_cash_balance",
                "insufficient_cash_flow",
                "insufficient_collateral",
                "insufficient_credit_experience",
                "insufficient_credit_utilization",
                "insufficient_deposits",
                "insufficient_income",
                "insufficient_margin_ratio",
                "insufficient_operating_profit",
                "insufficient_period_in_operation",
                "insufficient_reserves",
                "insufficient_revenue",
                "insufficient_social_media_performance",
                "insufficient_time_in_network",
                "insufficient_trade_credit_insurance",
                "insufficient_usage_as_qualified_expenses",
                "invalid_business_license",
                "lacking_cash_account",
                "late_payment_history_reported_to_bureau",
                "lien_collection_action_or_judgement",
                "negative_public_information",
                "no_credit_file",
                "other",
                "outside_supported_country",
                "outside_supported_state",
                "poor_payment_history_with_platform",
                "prior_or_current_legal_action",
                "prohibited_industry",
                "rate_of_cash_balance_fluctuation_too_high",
                "recent_inquiries_on_business_credit_report",
                "removal_of_bank_account_connection",
                "revenue_discrepancy",
                "runway_too_short",
                "suspected_fraud",
                "too_many_non_sufficient_funds_or_overdrafts",
                "unable_to_verify_address",
                "unable_to_verify_identity",
                "unable_to_verify_income_or_revenue",
                "unprofitable",
                "unsupportable_business_type",
            ]
        ]
        """
        List of reasons why the credit line was closed, up to 4 reasons, in order of importance.
        """

    class CorrectParamsUnderwritingException(TypedDict):
        explanation: str
        """
        Written explanation for the exception.
        """
        original_decision_type: Literal[
            "additional_information_requested",
            "application_rejected",
            "credit_limit_approved",
            "credit_limit_decreased",
            "credit_line_closed",
            "no_changes",
            "withdrawn_by_applicant",
        ]
        """
        The decision before the exception was applied.
        """

    class CreateFromApplicationParams(RequestOptions):
        application: (
            "CreditUnderwritingRecord.CreateFromApplicationParamsApplication"
        )
        """
        Details about the application submission.
        """
        credit_user: (
            "CreditUnderwritingRecord.CreateFromApplicationParamsCreditUser"
        )
        """
        Information about the company or person applying or holding the account.
        """
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        metadata: NotRequired[Dict[str, str]]
        """
        Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
        """

    class CreateFromApplicationParamsApplication(TypedDict):
        application_method: NotRequired[
            Literal["in_person", "mail", "online", "phone"]
        ]
        """
        The channel through which the applicant has submitted their application. Defaults to `online`.
        """
        purpose: Literal["credit_limit_increase", "credit_line_opening"]
        """
        Scope of demand made by the applicant.
        """
        submitted_at: int
        """
        Date when the applicant submitted their application.
        """

    class CreateFromApplicationParamsCreditUser(TypedDict):
        email: str
        """
        Email of the applicant or accountholder.
        """
        name: str
        """
        Full name of the company or person.
        """

    class CreateFromProactiveReviewParams(RequestOptions):
        credit_user: "CreditUnderwritingRecord.CreateFromProactiveReviewParamsCreditUser"
        """
        Information about the company or person applying or holding the account.
        """
        decided_at: int
        """
        Date when a decision was made.
        """
        decision: (
            "CreditUnderwritingRecord.CreateFromProactiveReviewParamsDecision"
        )
        """
        Details about the decision.
        """
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        metadata: NotRequired[Dict[str, str]]
        """
        Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
        """
        regulatory_reporting_file: NotRequired[str]
        """
        File containing regulatory reporting data for the decision. Required if you are subject to this [reporting requirement](https://stripe.com/docs/issuing/credit/report-required-regulatory-data-for-credit-decisions).
        """
        underwriting_exception: NotRequired[
            "CreditUnderwritingRecord.CreateFromProactiveReviewParamsUnderwritingException"
        ]
        """
        If an exception to the usual underwriting criteria was made for this decision, details about the exception must be provided. Exceptions should only be granted in rare circumstances, in consultation with Stripe Compliance.
        """

    class CreateFromProactiveReviewParamsCreditUser(TypedDict):
        email: str
        """
        Email of the applicant or accountholder.
        """
        name: str
        """
        Full name of the company or person.
        """

    class CreateFromProactiveReviewParamsDecision(TypedDict):
        credit_limit_approved: NotRequired[
            "CreditUnderwritingRecord.CreateFromProactiveReviewParamsDecisionCreditLimitApproved"
        ]
        """
        Details about the credit limit approved. An approved credit limit is required before you can set a `credit_limit_amount` in the [CreditPolicy API](https://stripe.com/docs/api/issuing/credit_policy/)
        """
        credit_limit_decreased: NotRequired[
            "CreditUnderwritingRecord.CreateFromProactiveReviewParamsDecisionCreditLimitDecreased"
        ]
        """
        Details about the credit limit decreased.
        """
        credit_line_closed: NotRequired[
            "CreditUnderwritingRecord.CreateFromProactiveReviewParamsDecisionCreditLineClosed"
        ]
        """
        Details about the credit line closed.
        """
        type: Literal[
            "credit_limit_approved",
            "credit_limit_decreased",
            "credit_line_closed",
            "no_changes",
        ]
        """
        Outcome of the decision.
        """

    class CreateFromProactiveReviewParamsDecisionCreditLimitApproved(
        TypedDict
    ):
        amount: int
        """
        The credit approved, in the currency of the account and [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
        """
        currency: NotRequired[str]
        """
        The currency of the credit approved, will default to the Account's Issuing currency.
        """

    class CreateFromProactiveReviewParamsDecisionCreditLimitDecreased(
        TypedDict,
    ):
        amount: int
        """
        The credit approved, in the currency of the account and [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
        """
        currency: NotRequired[str]
        """
        The currency of the credit approved, will default to the Account's Issuing currency.
        """
        reason_other_explanation: NotRequired[str]
        """
        Details about the `reasons.other` when present.
        """
        reasons: List[
            Literal[
                "applicant_is_not_beneficial_owner",
                "applicant_too_young",
                "application_is_not_beneficial_owner",
                "bankruptcy",
                "business_size_too_small",
                "change_in_financial_state",
                "change_in_utilization_of_credit_line",
                "current_account_tier_ineligible",
                "customer_already_exists",
                "customer_requested_account_closure",
                "debt_to_cash_balance_ratio_too_high",
                "debt_to_equity_ratio_too_high",
                "decrease_in_income_to_expense_ratio",
                "decrease_in_social_media_performance",
                "delinquent_credit_obligations",
                "dispute_rate_too_high",
                "duration_of_residence",
                "exceeds_acceptable_platform_exposure",
                "excessive_income_or_revenue_obligations",
                "expenses_to_cash_balance_ratio_too_high",
                "foreclosure_or_repossession",
                "frozen_file_at_credit_bureau",
                "garnishment_or_attachment",
                "government_loan_program_criteria",
                "has_recent_credit_limit_increase",
                "high_concentration_of_clients",
                "high_risk_industry",
                "incomplete_application",
                "inconsistent_monthly_revenues",
                "insufficient_account_history_with_platform",
                "insufficient_bank_account_history",
                "insufficient_cash_balance",
                "insufficient_cash_flow",
                "insufficient_collateral",
                "insufficient_credit_experience",
                "insufficient_credit_utilization",
                "insufficient_deposits",
                "insufficient_income",
                "insufficient_margin_ratio",
                "insufficient_operating_profit",
                "insufficient_period_in_operation",
                "insufficient_reserves",
                "insufficient_revenue",
                "insufficient_social_media_performance",
                "insufficient_time_in_network",
                "insufficient_trade_credit_insurance",
                "insufficient_usage_as_qualified_expenses",
                "invalid_business_license",
                "lacking_cash_account",
                "late_payment_history_reported_to_bureau",
                "lien_collection_action_or_judgement",
                "negative_public_information",
                "no_credit_file",
                "other",
                "outside_supported_country",
                "outside_supported_state",
                "poor_payment_history_with_platform",
                "prior_or_current_legal_action",
                "prohibited_industry",
                "rate_of_cash_balance_fluctuation_too_high",
                "recent_inquiries_on_business_credit_report",
                "removal_of_bank_account_connection",
                "revenue_discrepancy",
                "runway_too_short",
                "suspected_fraud",
                "too_many_non_sufficient_funds_or_overdrafts",
                "unable_to_verify_address",
                "unable_to_verify_identity",
                "unable_to_verify_income_or_revenue",
                "unprofitable",
                "unsupportable_business_type",
            ]
        ]
        """
        List of reasons why the existing credit was decreased, up to 4 reasons, in order of importance.
        """

    class CreateFromProactiveReviewParamsDecisionCreditLineClosed(TypedDict):
        reason_other_explanation: NotRequired[str]
        """
        Details about the `reasons.other` when present.
        """
        reasons: List[
            Literal[
                "applicant_is_not_beneficial_owner",
                "applicant_too_young",
                "application_is_not_beneficial_owner",
                "bankruptcy",
                "business_size_too_small",
                "change_in_financial_state",
                "change_in_utilization_of_credit_line",
                "current_account_tier_ineligible",
                "customer_already_exists",
                "customer_requested_account_closure",
                "debt_to_cash_balance_ratio_too_high",
                "debt_to_equity_ratio_too_high",
                "decrease_in_income_to_expense_ratio",
                "decrease_in_social_media_performance",
                "delinquent_credit_obligations",
                "dispute_rate_too_high",
                "duration_of_residence",
                "exceeds_acceptable_platform_exposure",
                "excessive_income_or_revenue_obligations",
                "expenses_to_cash_balance_ratio_too_high",
                "foreclosure_or_repossession",
                "frozen_file_at_credit_bureau",
                "garnishment_or_attachment",
                "government_loan_program_criteria",
                "has_recent_credit_limit_increase",
                "high_concentration_of_clients",
                "high_risk_industry",
                "incomplete_application",
                "inconsistent_monthly_revenues",
                "insufficient_account_history_with_platform",
                "insufficient_bank_account_history",
                "insufficient_cash_balance",
                "insufficient_cash_flow",
                "insufficient_collateral",
                "insufficient_credit_experience",
                "insufficient_credit_utilization",
                "insufficient_deposits",
                "insufficient_income",
                "insufficient_margin_ratio",
                "insufficient_operating_profit",
                "insufficient_period_in_operation",
                "insufficient_reserves",
                "insufficient_revenue",
                "insufficient_social_media_performance",
                "insufficient_time_in_network",
                "insufficient_trade_credit_insurance",
                "insufficient_usage_as_qualified_expenses",
                "invalid_business_license",
                "lacking_cash_account",
                "late_payment_history_reported_to_bureau",
                "lien_collection_action_or_judgement",
                "negative_public_information",
                "no_credit_file",
                "other",
                "outside_supported_country",
                "outside_supported_state",
                "poor_payment_history_with_platform",
                "prior_or_current_legal_action",
                "prohibited_industry",
                "rate_of_cash_balance_fluctuation_too_high",
                "recent_inquiries_on_business_credit_report",
                "removal_of_bank_account_connection",
                "revenue_discrepancy",
                "runway_too_short",
                "suspected_fraud",
                "too_many_non_sufficient_funds_or_overdrafts",
                "unable_to_verify_address",
                "unable_to_verify_identity",
                "unable_to_verify_income_or_revenue",
                "unprofitable",
                "unsupportable_business_type",
            ]
        ]
        """
        List of reasons why the credit line was closed, up to 4 reasons, in order of importance.
        """

    class CreateFromProactiveReviewParamsUnderwritingException(TypedDict):
        explanation: str
        """
        Written explanation for the exception.
        """
        original_decision_type: Literal[
            "additional_information_requested",
            "application_rejected",
            "credit_limit_approved",
            "credit_limit_decreased",
            "credit_line_closed",
            "no_changes",
            "withdrawn_by_applicant",
        ]
        """
        The decision before the exception was applied.
        """

    class ListParams(RequestOptions):
        ending_before: NotRequired[str]
        """
        A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
        """
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        limit: NotRequired[int]
        """
        A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
        """
        starting_after: NotRequired[str]
        """
        A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
        """

    class ReportDecisionParams(RequestOptions):
        decided_at: int
        """
        Date when a decision was made.
        """
        decision: "CreditUnderwritingRecord.ReportDecisionParamsDecision"
        """
        Details about the decision.
        """
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        metadata: NotRequired[Dict[str, str]]
        """
        Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
        """
        regulatory_reporting_file: NotRequired[str]
        """
        File containing regulatory reporting data for the decision. Required if you are subject to this [reporting requirement](https://stripe.com/docs/issuing/credit/report-required-regulatory-data-for-credit-decisions).
        """
        underwriting_exception: NotRequired[
            "CreditUnderwritingRecord.ReportDecisionParamsUnderwritingException"
        ]
        """
        If an exception to the usual underwriting criteria was made for this decision, details about the exception must be provided. Exceptions should only be granted in rare circumstances, in consultation with Stripe Compliance.
        """

    class ReportDecisionParamsDecision(TypedDict):
        application_rejected: NotRequired[
            "CreditUnderwritingRecord.ReportDecisionParamsDecisionApplicationRejected"
        ]
        """
        Details about the application rejection.
        """
        credit_limit_approved: NotRequired[
            "CreditUnderwritingRecord.ReportDecisionParamsDecisionCreditLimitApproved"
        ]
        """
        Details about the credit limit approved. An approved credit limit is required before you can set a `credit_limit_amount` in the [CreditPolicy API](https://stripe.com/docs/api/issuing/credit_policy/)
        """
        type: Literal[
            "additional_information_requested",
            "application_rejected",
            "credit_limit_approved",
            "withdrawn_by_applicant",
        ]
        """
        Outcome of the decision.
        """

    class ReportDecisionParamsDecisionApplicationRejected(TypedDict):
        reason_other_explanation: NotRequired[str]
        """
        Details about the `reasons.other` when present.
        """
        reasons: List[
            Literal[
                "applicant_is_not_beneficial_owner",
                "applicant_too_young",
                "application_is_not_beneficial_owner",
                "bankruptcy",
                "business_size_too_small",
                "current_account_tier_ineligible",
                "customer_already_exists",
                "customer_requested_account_closure",
                "debt_to_cash_balance_ratio_too_high",
                "debt_to_equity_ratio_too_high",
                "delinquent_credit_obligations",
                "dispute_rate_too_high",
                "duration_of_residence",
                "excessive_income_or_revenue_obligations",
                "expenses_to_cash_balance_ratio_too_high",
                "foreclosure_or_repossession",
                "frozen_file_at_credit_bureau",
                "garnishment_or_attachment",
                "government_loan_program_criteria",
                "high_concentration_of_clients",
                "high_risk_industry",
                "incomplete_application",
                "inconsistent_monthly_revenues",
                "insufficient_account_history_with_platform",
                "insufficient_bank_account_history",
                "insufficient_cash_balance",
                "insufficient_cash_flow",
                "insufficient_collateral",
                "insufficient_credit_experience",
                "insufficient_deposits",
                "insufficient_income",
                "insufficient_margin_ratio",
                "insufficient_operating_profit",
                "insufficient_period_in_operation",
                "insufficient_reserves",
                "insufficient_revenue",
                "insufficient_social_media_performance",
                "insufficient_time_in_network",
                "insufficient_trade_credit_insurance",
                "invalid_business_license",
                "lacking_cash_account",
                "late_payment_history_reported_to_bureau",
                "lien_collection_action_or_judgement",
                "negative_public_information",
                "no_credit_file",
                "other",
                "outside_supported_country",
                "outside_supported_state",
                "poor_payment_history_with_platform",
                "prior_or_current_legal_action",
                "prohibited_industry",
                "rate_of_cash_balance_fluctuation_too_high",
                "recent_inquiries_on_business_credit_report",
                "removal_of_bank_account_connection",
                "revenue_discrepancy",
                "runway_too_short",
                "suspected_fraud",
                "too_many_non_sufficient_funds_or_overdrafts",
                "unable_to_verify_address",
                "unable_to_verify_identity",
                "unable_to_verify_income_or_revenue",
                "unprofitable",
                "unsupportable_business_type",
            ]
        ]
        """
        List of reasons why the application was rejected, up to 4 reasons, in order of importance.
        """

    class ReportDecisionParamsDecisionCreditLimitApproved(TypedDict):
        amount: int
        """
        The credit approved, in the currency of the account and [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal).
        """
        currency: NotRequired[str]
        """
        The currency of the credit approved, will default to the Account's Issuing currency.
        """

    class ReportDecisionParamsUnderwritingException(TypedDict):
        explanation: str
        """
        Written explanation for the exception.
        """
        original_decision_type: Literal[
            "additional_information_requested",
            "application_rejected",
            "credit_limit_approved",
            "credit_limit_decreased",
            "credit_line_closed",
            "no_changes",
            "withdrawn_by_applicant",
        ]
        """
        The decision before the exception was applied.
        """

    class RetrieveParams(RequestOptions):
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """

    application: Optional[Application]
    """
    For decisions triggered by an application, details about the submission.
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    created_from: Literal["application", "proactive_review"]
    """
    The event that triggered the underwriting.
    """
    credit_user: CreditUser
    decided_at: Optional[int]
    """
    Date when a decision was made.
    """
    decision: Optional[Decision]
    """
    Details about the decision.
    """
    decision_deadline: Optional[int]
    """
    For underwriting initiated by an application, a decision must be taken 30 days after the submission.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    metadata: Optional[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["issuing.credit_underwriting_record"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    regulatory_reporting_file: Optional[str]
    """
    File containing regulatory reporting data for the decision. Required if you are subject to this [reporting requirement](https://stripe.com/docs/issuing/credit/report-required-regulatory-data-for-credit-decisions).
    """
    underwriting_exception: Optional[UnderwritingException]
    """
    If an exception to the usual underwriting criteria was made for this application, details about the exception must be provided. Exceptions should only be granted in rare circumstances, in consultation with Stripe Compliance.
    """

    @classmethod
    def _cls_correct(
        cls,
        credit_underwriting_record: str,
        **params: Unpack["CreditUnderwritingRecord.CorrectParams"],
    ) -> "CreditUnderwritingRecord":
        """
        Update a CreditUnderwritingRecord object to correct mistakes.
        """
        return cast(
            "CreditUnderwritingRecord",
            cls._static_request(
                "post",
                "/v1/issuing/credit_underwriting_records/{credit_underwriting_record}/correct".format(
                    credit_underwriting_record=sanitize_id(
                        credit_underwriting_record
                    )
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def correct(
        credit_underwriting_record: str,
        **params: Unpack["CreditUnderwritingRecord.CorrectParams"],
    ) -> "CreditUnderwritingRecord":
        """
        Update a CreditUnderwritingRecord object to correct mistakes.
        """
        ...

    @overload
    def correct(
        self, **params: Unpack["CreditUnderwritingRecord.CorrectParams"]
    ) -> "CreditUnderwritingRecord":
        """
        Update a CreditUnderwritingRecord object to correct mistakes.
        """
        ...

    @class_method_variant("_cls_correct")
    def correct(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["CreditUnderwritingRecord.CorrectParams"]
    ) -> "CreditUnderwritingRecord":
        """
        Update a CreditUnderwritingRecord object to correct mistakes.
        """
        return cast(
            "CreditUnderwritingRecord",
            self._request(
                "post",
                "/v1/issuing/credit_underwriting_records/{credit_underwriting_record}/correct".format(
                    credit_underwriting_record=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_correct_async(
        cls,
        credit_underwriting_record: str,
        **params: Unpack["CreditUnderwritingRecord.CorrectParams"],
    ) -> "CreditUnderwritingRecord":
        """
        Update a CreditUnderwritingRecord object to correct mistakes.
        """
        return cast(
            "CreditUnderwritingRecord",
            await cls._static_request_async(
                "post",
                "/v1/issuing/credit_underwriting_records/{credit_underwriting_record}/correct".format(
                    credit_underwriting_record=sanitize_id(
                        credit_underwriting_record
                    )
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def correct_async(
        credit_underwriting_record: str,
        **params: Unpack["CreditUnderwritingRecord.CorrectParams"],
    ) -> "CreditUnderwritingRecord":
        """
        Update a CreditUnderwritingRecord object to correct mistakes.
        """
        ...

    @overload
    async def correct_async(
        self, **params: Unpack["CreditUnderwritingRecord.CorrectParams"]
    ) -> "CreditUnderwritingRecord":
        """
        Update a CreditUnderwritingRecord object to correct mistakes.
        """
        ...

    @class_method_variant("_cls_correct_async")
    async def correct_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["CreditUnderwritingRecord.CorrectParams"]
    ) -> "CreditUnderwritingRecord":
        """
        Update a CreditUnderwritingRecord object to correct mistakes.
        """
        return cast(
            "CreditUnderwritingRecord",
            await self._request_async(
                "post",
                "/v1/issuing/credit_underwriting_records/{credit_underwriting_record}/correct".format(
                    credit_underwriting_record=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def create_from_application(
        cls,
        **params: Unpack[
            "CreditUnderwritingRecord.CreateFromApplicationParams"
        ],
    ) -> "CreditUnderwritingRecord":
        """
        Creates a CreditUnderwritingRecord object with information about a credit application submission.
        """
        return cast(
            "CreditUnderwritingRecord",
            cls._static_request(
                "post",
                "/v1/issuing/credit_underwriting_records/create_from_application",
                params=params,
            ),
        )

    @classmethod
    async def create_from_application_async(
        cls,
        **params: Unpack[
            "CreditUnderwritingRecord.CreateFromApplicationParams"
        ],
    ) -> "CreditUnderwritingRecord":
        """
        Creates a CreditUnderwritingRecord object with information about a credit application submission.
        """
        return cast(
            "CreditUnderwritingRecord",
            await cls._static_request_async(
                "post",
                "/v1/issuing/credit_underwriting_records/create_from_application",
                params=params,
            ),
        )

    @classmethod
    def create_from_proactive_review(
        cls,
        **params: Unpack[
            "CreditUnderwritingRecord.CreateFromProactiveReviewParams"
        ],
    ) -> "CreditUnderwritingRecord":
        """
        Creates a CreditUnderwritingRecord object from an underwriting decision coming from a proactive review of an existing accountholder.
        """
        return cast(
            "CreditUnderwritingRecord",
            cls._static_request(
                "post",
                "/v1/issuing/credit_underwriting_records/create_from_proactive_review",
                params=params,
            ),
        )

    @classmethod
    async def create_from_proactive_review_async(
        cls,
        **params: Unpack[
            "CreditUnderwritingRecord.CreateFromProactiveReviewParams"
        ],
    ) -> "CreditUnderwritingRecord":
        """
        Creates a CreditUnderwritingRecord object from an underwriting decision coming from a proactive review of an existing accountholder.
        """
        return cast(
            "CreditUnderwritingRecord",
            await cls._static_request_async(
                "post",
                "/v1/issuing/credit_underwriting_records/create_from_proactive_review",
                params=params,
            ),
        )

    @classmethod
    def list(
        cls, **params: Unpack["CreditUnderwritingRecord.ListParams"]
    ) -> ListObject["CreditUnderwritingRecord"]:
        """
        Retrieves a list of CreditUnderwritingRecord objects. The objects are sorted in descending order by creation date, with the most-recently-created object appearing first.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["CreditUnderwritingRecord.ListParams"]
    ) -> ListObject["CreditUnderwritingRecord"]:
        """
        Retrieves a list of CreditUnderwritingRecord objects. The objects are sorted in descending order by creation date, with the most-recently-created object appearing first.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def _cls_report_decision(
        cls,
        credit_underwriting_record: str,
        **params: Unpack["CreditUnderwritingRecord.ReportDecisionParams"],
    ) -> "CreditUnderwritingRecord":
        """
        Update a CreditUnderwritingRecord object from a decision made on a credit application.
        """
        return cast(
            "CreditUnderwritingRecord",
            cls._static_request(
                "post",
                "/v1/issuing/credit_underwriting_records/{credit_underwriting_record}/report_decision".format(
                    credit_underwriting_record=sanitize_id(
                        credit_underwriting_record
                    )
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def report_decision(
        credit_underwriting_record: str,
        **params: Unpack["CreditUnderwritingRecord.ReportDecisionParams"],
    ) -> "CreditUnderwritingRecord":
        """
        Update a CreditUnderwritingRecord object from a decision made on a credit application.
        """
        ...

    @overload
    def report_decision(
        self, **params: Unpack["CreditUnderwritingRecord.ReportDecisionParams"]
    ) -> "CreditUnderwritingRecord":
        """
        Update a CreditUnderwritingRecord object from a decision made on a credit application.
        """
        ...

    @class_method_variant("_cls_report_decision")
    def report_decision(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["CreditUnderwritingRecord.ReportDecisionParams"]
    ) -> "CreditUnderwritingRecord":
        """
        Update a CreditUnderwritingRecord object from a decision made on a credit application.
        """
        return cast(
            "CreditUnderwritingRecord",
            self._request(
                "post",
                "/v1/issuing/credit_underwriting_records/{credit_underwriting_record}/report_decision".format(
                    credit_underwriting_record=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_report_decision_async(
        cls,
        credit_underwriting_record: str,
        **params: Unpack["CreditUnderwritingRecord.ReportDecisionParams"],
    ) -> "CreditUnderwritingRecord":
        """
        Update a CreditUnderwritingRecord object from a decision made on a credit application.
        """
        return cast(
            "CreditUnderwritingRecord",
            await cls._static_request_async(
                "post",
                "/v1/issuing/credit_underwriting_records/{credit_underwriting_record}/report_decision".format(
                    credit_underwriting_record=sanitize_id(
                        credit_underwriting_record
                    )
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def report_decision_async(
        credit_underwriting_record: str,
        **params: Unpack["CreditUnderwritingRecord.ReportDecisionParams"],
    ) -> "CreditUnderwritingRecord":
        """
        Update a CreditUnderwritingRecord object from a decision made on a credit application.
        """
        ...

    @overload
    async def report_decision_async(
        self, **params: Unpack["CreditUnderwritingRecord.ReportDecisionParams"]
    ) -> "CreditUnderwritingRecord":
        """
        Update a CreditUnderwritingRecord object from a decision made on a credit application.
        """
        ...

    @class_method_variant("_cls_report_decision_async")
    async def report_decision_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["CreditUnderwritingRecord.ReportDecisionParams"]
    ) -> "CreditUnderwritingRecord":
        """
        Update a CreditUnderwritingRecord object from a decision made on a credit application.
        """
        return cast(
            "CreditUnderwritingRecord",
            await self._request_async(
                "post",
                "/v1/issuing/credit_underwriting_records/{credit_underwriting_record}/report_decision".format(
                    credit_underwriting_record=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls,
        id: str,
        **params: Unpack["CreditUnderwritingRecord.RetrieveParams"],
    ) -> "CreditUnderwritingRecord":
        """
        Retrieves a CreditUnderwritingRecord object.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls,
        id: str,
        **params: Unpack["CreditUnderwritingRecord.RetrieveParams"],
    ) -> "CreditUnderwritingRecord":
        """
        Retrieves a CreditUnderwritingRecord object.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {
        "application": Application,
        "credit_user": CreditUser,
        "decision": Decision,
        "underwriting_exception": UnderwritingException,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._coupon import Coupon
from stripe._list_object import ListObject
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Dict, List, cast
from typing_extensions import Literal, NotRequired, TypedDict


class CouponService(StripeService):
    class CreateParams(TypedDict):
        amount_off: NotRequired[int]
        """
        A positive integer representing the amount to subtract from an invoice total (required if `percent_off` is not passed).
        """
        applies_to: NotRequired["CouponService.CreateParamsAppliesTo"]
        """
        A hash containing directions for what this Coupon will apply discounts to.
        """
        currency: NotRequired[str]
        """
        Three-letter [ISO code for the currency](https://stripe.com/docs/currencies) of the `amount_off` parameter (required if `amount_off` is passed).
        """
        currency_options: NotRequired[
            Dict[str, "CouponService.CreateParamsCurrencyOptions"]
        ]
        """
        Coupons defined in each available currency option (only supported if `amount_off` is passed). Each key must be a three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html) and a [supported currency](https://stripe.com/docs/currencies).
        """
        duration: NotRequired[
            Literal["forever", "once", "repeating", "variable"]
        ]
        """
        Specifies how long the discount will be in effect if used on a subscription. Defaults to `once`.
        """
        duration_in_months: NotRequired[int]
        """
        Required only if `duration` is `repeating`, in which case it must be a positive integer that specifies the number of months the discount will be in effect.
        """
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        id: NotRequired[str]
        """
        Unique string of your choice that will be used to identify this coupon when applying it to a customer. If you don't want to specify a particular code, you can leave the ID blank and we'll generate a random code for you.
        """
        max_redemptions: NotRequired[int]
        """
        A positive integer specifying the number of times the coupon can be redeemed before it's no longer valid. For example, you might have a 50% off coupon that the first 20 readers of your blog can use.
        """
        metadata: NotRequired["Literal['']|Dict[str, str]"]
        """
        Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
        """
        name: NotRequired[str]
        """
        Name of the coupon displayed to customers on, for instance invoices, or receipts. By default the `id` is shown if `name` is not set.
        """
        percent_off: NotRequired[float]
        """
        A positive float larger than 0, and smaller or equal to 100, that represents the discount the coupon will apply (required if `amount_off` is not passed).
        """
        redeem_by: NotRequired[int]
        """
        Unix timestamp specifying the last time at which the coupon can be redeemed. After the redeem_by date, the coupon can no longer be applied to new customers.
        """

    class CreateParamsAppliesTo(TypedDict):
        products: NotRequired[List[str]]
        """
        An array of Product IDs that this Coupon will apply to.
        """

    class CreateParamsCurrencyOptions(TypedDict):
        amount_off: int
        """
        A positive integer representing the amount to subtract from an invoice total.
        """

    class DeleteParams(TypedDict):
        pass

    class ListParams(TypedDict):
        created: NotRequired["CouponService.ListParamsCreated|int"]
        """
        A filter on the list, based on the object `created` field. The value can be a string with an integer Unix timestamp, or it can be a dictionary with a number of different query options.
        """
        ending_before: NotRequired[str]
        """
        A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
        """
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        limit: NotRequired[int]
        """
        A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
        """
        starting_after: NotRequired[str]
        """
        A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
        """

    class ListParamsCreated(TypedDict):
        gt: NotRequired[int]
        """
        Minimum value to filter by (exclusive)
        """
        gte: NotRequired[int]
        """
        Minimum value to filter by (inclusive)
        """
        lt: NotRequired[int]
        """
        Maximum value to filter by (exclusive)
        """
        lte: NotRequired[int]
        """
        Maximum value to filter by (inclusive)
        """

    class RetrieveParams(TypedDict):
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """

    class UpdateParams(TypedDict):
        currency_options: NotRequired[
            Dict[str, "CouponService.UpdateParamsCurrencyOptions"]
        ]
        """
        Coupons defined in each available currency option (only supported if the coupon is amount-based). Each key must be a three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html) and a [supported currency](https://stripe.com/docs/currencies).
        """
        expand: NotRequired[List[str]]
        """
        Specifies which fields in the response should be expanded.
        """
        metadata: NotRequired["Literal['']|Dict[str, str]"]
        """
        Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
        """
        name: NotRequired[str]
        """
        Name of the coupon displayed to customers on, for instance invoices, or receipts. By default the `id` is shown if `name` is not set.
        """

    class UpdateParamsCurrencyOptions(TypedDict):
        amount_off: int
        """
        A positive integer representing the amount to subtract from an invoice total.
        """

    def delete(
        self,
        coupon: str,
        params: "CouponService.DeleteParams" = {},
        options: RequestOptions = {},
    ) -> Coupon:
        """
        You can delete coupons via the [coupon management](https://dashboard.stripe.com/coupons) page of the Stripe dashboard. However, deleting a coupon does not affect any customers who have already applied the coupon; it means that new customers can't redeem the coupon. You can also delete coupons via the API.
        """
        return cast(
            Coupon,
            self._request(
                "delete",
                "/v1/coupons/{coupon}".format(coupon=sanitize_id(coupon)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def delete_async(
        self,
        coupon: str,
        params: "CouponService.DeleteParams" = {},
        options: RequestOptions = {},
    ) -> Coupon:
        """
        You can delete coupons via the [coupon management](https://dashboard.stripe.com/coupons) page of the Stripe dashboard. However, deleting a coupon does not affect any customers who have already applied the coupon; it means that new customers can't redeem the coupon. You can also delete coupons via the API.
        """
        return cast(
            Coupon,
            await self._request_async(
                "delete",
                "/v1/coupons/{coupon}".format(coupon=sanitize_id(coupon)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        coupon: str,
        params: "CouponService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> Coupon:
        """
        Retrieves the coupon with the given ID.
        """
        return cast(
            Coupon,
            self._request(
                "get",
                "/v1/coupons/{coupon}".format(coupon=sanitize_id(coupon)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        coupon: str,
        params: "CouponService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> Coupon:
        """
        Retrieves the coupon with the given ID.
        """
        return cast(
            Coupon,
            await self._request_async(
                "get",
                "/v1/coupons/{coupon}".format(coupon=sanitize_id(coupon)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        coupon: str,
        params: "CouponService.UpdateParams" = {},
        options: RequestOptions = {},
    ) -> Coupon:
        """
        Updates the metadata of a coupon. Other coupon details (currency, duration, amount_off) are, by design, not editable.
        """
        return cast(
            Coupon,
            self._request(
                "post",
                "/v1/coupons/{coupon}".format(coupon=sanitize_id(coupon)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        coupon: str,
        params: "CouponService.UpdateParams" = {},
        options: RequestOptions = {},
    ) -> Coupon:
        """
        Updates the metadata of a coupon. Other coupon details (currency, duration, amount_off) are, by design, not editable.
        """
        return cast(
            Coupon,
            await self._request_async(
                "post",
                "/v1/coupons/{coupon}".format(coupon=sanitize_id(coupon)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def list(
        self,
        params: "CouponService.ListParams" = {},
        options: RequestOptions = {},
    ) -> ListObject[Coupon]:
        """
        Returns a list of your coupons.
        """
        return cast(
            ListObject[Coupon],
            self._request(
                "get",
                "/v1/coupons",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: "CouponService.ListParams" = {},
        options: RequestOptions = {},
    ) -> ListObject[Coupon]:
        """
        Returns a list of your coupons.
        """
        return cast(
            ListObject[Coupon],
            await self._request_async(
                "get",
                "/v1/coupons",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "CouponService.CreateParams" = {},
        options: RequestOptions = {},
    ) -> Coupon:
        """
        You can create coupons easily via the [coupon management](https://dashboard.stripe.com/coupons) page of the Stripe dashboard. Coupon creation is also accessible via the API if you need to create coupons on the fly.

        A coupon has either a percent_off or an amount_off and currency. If you set an amount_off, that amount will be subtracted from any invoice's subtotal. For example, an invoice with a subtotal of 100 will have a final total of 0 if a coupon with an amount_off of 200 is applied to it and an invoice with a subtotal of 300 will have a final total of 100 if a coupon with an amount_off of 200 is applied to it.
        """
        return cast(
            Coupon,
            self._request(
                "post",
                "/v1/coupons",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "CouponService.CreateParams" = {},
        options: RequestOptions = {},
    ) -> Coupon:
        """
        You can create coupons easily via the [coupon management](https://dashboard.stripe.com/coupons) page of the Stripe dashboard. Coupon creation is also accessible via the API if you need to create coupons on the fly.

        A coupon has either a percent_off or an amount_off and currency. If you set an amount_off, that amount will be subtracted from any invoice's subtotal. For example, an invoice with a subtotal of 100 will have a final total of 0 if a coupon with an amount_off of 200 is applied to it and an invoice with a subtotal of 300 will have a final total of 100 if a coupon with an amount_off of 200 is applied to it.
        """
        return cast(
            Coupon,
            await self._request_async(
                "post",
                "/v1/coupons",
                base_address="api",
                params=params,
                options=options,
            ),
        )
